/**
	@file 	dialogs.c
	@brief 	Dialog box callback functions
*/
/*
pGina - A modular GINA Interface for Windows
Copyright (C) 2002 Nathan Yocom, Michael Wright

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Email: nate.yocom@xpasystems.com
Web: http://pgina.xpasystems.com
Snail Mail:
  Nathan Yocom 
  9 Evergreen Farms Rd.
  Scarborough, ME 04074
  Phone: 207-450-4948
*/
/// FOR FUTURE REFERENCE, WINDOWS CONTROLS AND MESSAGE HERE: http://msdn.microsoft.com/library/en-us/shellcc/platform/commctls/indivcontrol.asp

#include <windows.h>
#include <winwlx.h>
#include "pGina.h"
#include "resource.h"

#include <Dsgetdc.h>

int dwGetDomainErr = -1;
LPTSTR szDomainNameNetBIOS=NULL;
LPTSTR szDomainNameDNS=NULL;
TCHAR   szComputerName[MAX_COMPUTERNAME_LENGTH + 1];
BOOL autoLoggedIn = FALSE;			// are we attempting an autologin?

LPCTSTR getComputerName()
{
	DWORD sz = sizeof(szComputerName);
	GetComputerName(szComputerName,&sz);
	return szComputerName;
}

void getDCInfo()
{
	if (dwGetDomainErr==-1){
		PDS_DOMAIN_TRUSTS pDomains;
		DWORD dwCount=0;

		szDomainNameNetBIOS=NULL;
		szDomainNameDNS=NULL;

		dwGetDomainErr = DsEnumerateDomainTrusts(NULL,DS_DOMAIN_PRIMARY,&pDomains,&dwCount);

		if ( dwGetDomainErr != NO_ERROR ){
			dwGetDomainErr = -1;
			return;
		}
		if (dwCount == 0){
			return;
		}
		szDomainNameNetBIOS = _tcsdup( pDomains->NetbiosDomainName  );
		szDomainNameDNS     = _tcsdup( pDomains->DnsDomainName     );

		NetApiBufferFree(pDomains);
	}
}

LPTSTR getDomain()
{
	getDCInfo();
	return szDomainNameNetBIOS;	
}

LPCTSTR analyzeSecs( int totalsecs)
{
	static TCHAR buf[256];
	int hours,mins,secs;
	secs  = totalsecs; 
	mins  = secs/60; 
	hours = mins/60;
	secs = secs%60;	
	mins = mins%60;
	
	if ( hours == 0 ) _stprintf( buf, _T("%d:%02d") ,mins, secs );
	else _stprintf( buf, _T("%d:%02d:%02d"),hours, mins, secs );
	
	return buf;
}

LPCTSTR idleSecs( int totalsecs)
{
	static TCHAR buf[256];
	int hours,mins,secs;
	secs  = totalsecs; 
	mins  = secs/60; 
	hours = mins/60;
	secs = secs%60;	
	mins = mins%60;
	
	if ( hours == 0 ) _stprintf( buf, _T("%d:%02d") ,mins, secs );
	else _stprintf( buf, _T("%d:%02d:%02d"),hours, mins, secs );
	
	return buf;
}

DWORD DlgTimerID;
DWORD idleDlgTimerID;

VOID CALLBACK onDlgTimerProc(          HWND hwndDlg,
    UINT uMsg,
    UINT idEvent,
    DWORD dwTime
)
{
    BOOL	sessionExpired;

    if (sessionTimeLeft == 0){
    	ShowWindow(GetDlgItem(hwndDlg,IDC_SESSIONTIME),FALSE);
    }else if (sessionTimeLeft > 0){
    	ShowWindow(GetDlgItem(hwndDlg,IDC_SESSIONTIME),TRUE);
    	SetDlgItemText(hwndDlg,IDC_SESSIONTIME,analyzeSecs(sessionTimeLeft));
	}else{
    	ShowWindow(GetDlgItem(hwndDlg,IDC_SESSIONTIME),TRUE);
    	SetDlgItemText(hwndDlg,IDC_SESSIONTIME,_T("Session expired! "));
	}

    sessionExpired  = (sessionTimeLeft < 0) ; 

    EnableWindow(GetDlgItem(hwndDlg,IDC_LOCK),!sessionExpired && allowLocking);
	EnableWindow(GetDlgItem(hwndDlg,ID_CHANGEPWD),!sessionExpired && settingsInfo.allowPassChange);
    EnableWindow(GetDlgItem(hwndDlg,ID_TASK),!sessionExpired && enableTask);
    EnableWindow(GetDlgItem(hwndDlg,ID_LOGOUT),sessionExpired || enableLogout);
    EnableWindow(GetDlgItem(hwndDlg,ID_RESTART),enableRestart);
	EnableWindow(GetDlgItem(hwndDlg,ID_CHANGEPWD),!sessionExpired && !disablePassword);
	EnableWindow(GetDlgItem(hwndDlg,IDCANCEL),!sessionExpired);
}

int idleTimeLeft;

// Callback when showing the idle dialog, countdown the time to forcing a logoff
VOID CALLBACK idleDlgTimerProc(HWND hwndDlg,UINT uMsg,UINT idEvent,DWORD dwTime) {
    // Get whats currently shown, then decrement
    wchar_t buffer[2048];
    idleTimeLeft--;
    if(idleTimeLeft >= 0) {
        wsprintf(buffer,TEXT("%s"),idleSecs(idleTimeLeft));
        SetDlgItemText(hwndDlg,IDC_SECONDS,buffer);
    }
    if(idleTimeLeft == 0) {
        SendMessage(hwndDlg,WM_COMMAND,IDLE_MESSAGE_TIMEOUT,0);
    }
}


/**
	@brief	Callback function for the Logged On CTRL+ALT+DEL dialog
	@author	Nathan Yocom
	
	Handles letting user change password, or lock workstation when
	then press CTRL+ALT+DEL while logged in.  If they are not allowed
	to lock, or change their password, the relevant buttons are disabled,
	otherwise we lock the workstation, or call the ChangePassword() dialog
	box.
*/
BOOL CALLBACK LockedBoxProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{
    PTCHAR  pszSystemName;
	BOOL	sessionExpired;
    LPTSTR Section = TEXT("IDD_LOCKDIAG");

	switch(message)
	{
	case WM_INITDIALOG:
		CenterWindow(hwndDlg);

		// Translations
		SetWindowText(hwndDlg,TranslatedString(Section,TEXT("Caption")));
		SetDlgItemText(hwndDlg,IDC_1,TranslatedString(Section,TEXT("1")));
		SetDlgItemText(hwndDlg,IDC_2,TranslatedString(Section,TEXT("2")));
		SetDlgItemText(hwndDlg,IDC_3,TranslatedString(Section,TEXT("3")));
		SetDlgItemText(hwndDlg,IDC_LOCK,TranslatedString(Section,TEXT("4")));
		SetDlgItemText(hwndDlg,ID_CHANGEPWD,TranslatedString(Section,TEXT("5")));
		SetDlgItemText(hwndDlg,ID_TASK,TranslatedString(Section,TEXT("6")));
		SetDlgItemText(hwndDlg,IDC_7,TranslatedString(Section,TEXT("7")));
		SetDlgItemText(hwndDlg,ID_LOGOUT,TranslatedString(Section,TEXT("Logout")));
		SetDlgItemText(hwndDlg,ID_RESTART,TranslatedString(Section,TEXT("Restart")));
		SetDlgItemText(hwndDlg,IDCANCEL,TranslatedString(Section,TEXT("Cancel")));

        if(showUsernameAndAuth)
            SetDlgItemText(hwndDlg,IDC_LOGONNAME,settingsInfo.Username);
        else
            SetDlgItemText(hwndDlg,IDC_LOGONNAME,TEXT("*********"));

		SetDlgItemText(hwndDlg,IDC_ABOUTTEXT,VERSION);

        if(showUsernameAndAuth) {
        
		if(settingsInfo.authType == AUTH_PLUGIN)
			SetDlgItemText(hwndDlg,IDC_AUTHBY,TEXT("By pGina Plugin"));
		else if(settingsInfo.authType == AUTH_LOCAL)
			SetDlgItemText(hwndDlg,IDC_AUTHBY,TEXT("As Local User"));
		else
			SetDlgItemText(hwndDlg,IDC_AUTHBY,TEXT("As Domain User"));
        } else {
            SetDlgItemText(hwndDlg,IDC_AUTHBY,TEXT("**********"));
        }

        sessionExpired  = (sessionTimeLeft < 0);
		
		EnableWindow(GetDlgItem(hwndDlg,IDC_LOCK),!sessionExpired && (allowLocking || (settingsInfo.isAdmin && regReadBool(TEXT("allowAdminLocks")))));
		EnableWindow(GetDlgItem(hwndDlg,ID_CHANGEPWD),!sessionExpired && settingsInfo.allowPassChange);
		EnableWindow(GetDlgItem(hwndDlg,ID_TASK),!sessionExpired && enableTask);
		EnableWindow(GetDlgItem(hwndDlg,ID_LOGOUT),sessionExpired || enableLogout);
		EnableWindow(GetDlgItem(hwndDlg,ID_RESTART),enableRestart);
		EnableWindow(GetDlgItem(hwndDlg,ID_CHANGEPWD),!disablePassword);
		EnableWindow(GetDlgItem(hwndDlg,ID_CHANGEPWD),!sessionExpired && !disablePassword);
		EnableWindow(GetDlgItem(hwndDlg,IDCANCEL),!sessionExpired);

		DlgTimerID = 0;
		    	
		if (sessionTimeLeft == 0) {
			ShowWindow(GetDlgItem(hwndDlg,IDC_SESSIONTIME),FALSE);
			ShowWindow(GetDlgItem(hwndDlg,IDC_7),FALSE);
		}else if (sessionTimeLeft > 0){
			ShowWindow(GetDlgItem(hwndDlg,IDC_SESSIONTIME),TRUE);
			ShowWindow(GetDlgItem(hwndDlg,IDC_7),TRUE);
		 	MakeBold(GetDlgItem(hwndDlg, IDC_SESSIONTIME), FALSE, FW_BOLD);
			SetDlgItemText(hwndDlg,IDC_SESSIONTIME,analyzeSecs(sessionTimeLeft));
			DlgTimerID = SetTimer(hwndDlg,0,1000,onDlgTimerProc);
		}else{
			ShowWindow(GetDlgItem(hwndDlg,IDC_SESSIONTIME),TRUE);
			ShowWindow(GetDlgItem(hwndDlg,IDC_7),TRUE);
		 	MakeBold(GetDlgItem(hwndDlg, IDC_SESSIONTIME), FALSE, FW_BOLD);
			SetDlgItemText(hwndDlg,IDC_SESSIONTIME,TranslatedString(Section,TEXT("8")));
		}

        if(showMachineName) {
            pszSystemName = GlobalAlloc( GPTR, 2048 );
            GetEnvironmentVariable( _T("COMPUTERNAME"), pszSystemName, 2048 );
            SetDlgItemText(hwndDlg, IDC_MACHINENAME, pszSystemName);
            GlobalFree(pszSystemName);
        } else {
            SetDlgItemText(hwndDlg, IDC_MACHINENAME, TEXT("**********"));
        }

        SetFocus(GetDlgItem(hwndDlg, IDC_LOCK));

        return TRUE;

	case WM_DESTROY:
	    if (DlgTimerID !=0) KillTimer(hwndDlg,DlgTimerID);
		return TRUE;

	case WM_COMMAND: 
		switch(LOWORD(wParam))
		{
		// User has entered something and hit OK
		case ID_CHANGEPWD:
			// Display change password dialog box if its a plugin user
            DialogBox(pGinahInstance,MAKEINTRESOURCE(IDD_PASSWD),NULL,(DLGPROC)PasswordBoxProc);    
            break;
		case IDC_LOCK:
			EndDialog(hwndDlg,WLX_SAS_ACTION_LOCK_WKSTA); 
			break;
		case IDCANCEL:
			EndDialog(hwndDlg, WLX_SAS_ACTION_NONE);
			break;
		case ID_LOGOUT:
			EndDialog(hwndDlg, WLX_SAS_ACTION_LOGOFF);
			break;
		case ID_TASK:
			EndDialog(hwndDlg, WLX_SAS_ACTION_TASKLIST);
			break;
		case ID_RESTART:
			EndDialog(hwndDlg, WLX_SAS_ACTION_SHUTDOWN_REBOOT);
			break;
		}
		return TRUE;
	} 
	return FALSE; 
}

BOOL CALLBACK IdleBoxProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{
    wchar_t buffer[2048];
    LPTSTR Section = TEXT("IDD_IDLELOGOUT");

    LoadSettings();

	switch(message)
	{
	case WM_INITDIALOG:
		CenterWindow(hwndDlg);
        if(!idleInterrupted) {
            idleTimeLeft = idleTime;
            if (idleTime == 0) {
                EndDialog(hwndDlg,IDLE_LOGOFF);
                return TRUE;
            }
        }
        else
            idleInterrupted = false;

		// Translations
		SetWindowText(hwndDlg,TranslatedString(Section,TEXT("Caption")));
		SetDlgItemText(hwndDlg,IDC_WARNING,TranslatedString(Section,TEXT("Warning")));
        SetDlgItemText(hwndDlg,IDC_TIMEREMAINING,TranslatedString(Section,TEXT("Time")));
        SetDlgItemText(hwndDlg,IDC_LOGOFF,TranslatedString(Section,TEXT("Logoff")));
        SetDlgItemText(hwndDlg,IDC_CANCEL,TranslatedString(Section,TEXT("Cancel")));

		EnableWindow(GetDlgItem(hwndDlg,IDC_CANCEL),idleAllowReturn);
		EnableWindow(GetDlgItem(hwndDlg,IDC_LOCK),idleAllowLock);

        wsprintf(buffer,TEXT("%s"),idleSecs(idleTimeLeft));
        SetDlgItemText(hwndDlg,IDC_SECONDS,buffer);

   		idleDlgTimerID = 0;
		    	
   		idleDlgTimerID = SetTimer(hwndDlg,0,1000,idleDlgTimerProc);
        return TRUE;

	case WM_DESTROY:
	    if (idleDlgTimerID !=0) KillTimer(hwndDlg,idleDlgTimerID);
		return TRUE;

	case WM_COMMAND: 
		switch(LOWORD(wParam))
		{
		// User has hit a button
        case IDC_LOGOFF:
        case IDLE_MESSAGE_TIMEOUT:
            EndDialog(hwndDlg,IDLE_LOGOFF);
			break;
        case IDC_CANCEL:
            EndDialog(hwndDlg,IDLE_CANCEL);
            break;
		}
		return TRUE;
	} 
	return FALSE; 
}


/**
	@brief	Callback function for the change password dialog box
	@author	Nathan Yocom

	Handles letting user change password, if the plugins ChangePassword() fails
	then we don't change the local user's password, otherwise we use the NetAPI
	to force a change in their local password.
*/
BOOL CALLBACK PasswordBoxProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{
	LPTSTR oldPass;
	LPTSTR newPass;
	LPTSTR verify;
	USER_INFO_1003 ui;
	NET_API_STATUS nStatus;
	LPTSTR domainToUse;
	LPTSTR serverToUse = NULL;
	HANDLE hUser;
	TCHAR TempString[1024];

	LPTSTR Section = TEXT("IDD_PASSWD");

	switch(message)
	{
	case WM_INITDIALOG:
		CenterWindow(hwndDlg);

		// Translations
		SetWindowText(hwndDlg, TranslatedString(Section,TEXT("Caption")));
		SetDlgItemText(hwndDlg, IDC_1,TranslatedString(Section,TEXT("1"))); 
		SetDlgItemText(hwndDlg, IDC_2,TranslatedString(Section,TEXT("2"))); 
		SetDlgItemText(hwndDlg, IDC_3,TranslatedString(Section,TEXT("3"))); 
		SetDlgItemText(hwndDlg, IDOK,TranslatedString(Section,TEXT("OK"))); 
		SetDlgItemText(hwndDlg, IDCANCEL,TranslatedString(Section,TEXT("Cancel"))); 


		if(!settingsInfo.allowPassChange && settingsInfo.authType == AUTH_PLUGIN)
		{
			// "You are not allowed to change your password"
			// pGinaMessageBox(IDS_PWDNOCHANGE,IDS_T_NOPE,MB_OK);
			MessageBox(GetForegroundWindow(),TranslatedString(Section,TEXT("IDS_PWDNOCHANGE")),NULL,MB_OK);
			EndDialog(hwndDlg,IDCANCEL);
			break;
		}
		else
		{
			SetFocus(GetDlgItem(hwndDlg, IDC_OLDPWD));
			return TRUE;
		}

	case WM_COMMAND: 
		switch(LOWORD(wParam))
		{
		// User has entered something and hit OK
		case IDOK:
			GetDlgItemText(hwndDlg,IDC_OLDPWD,TempString,sizeof(TempString)/2-1);
			oldPass = _tcsdup(TempString);
			GetDlgItemText(hwndDlg,IDC_NEWPWD1,TempString,sizeof(TempString)/2-1);
			newPass = _tcsdup(TempString);
			GetDlgItemText(hwndDlg,IDC_NEWPWD2,TempString,sizeof(TempString)/2-1);
			verify = _tcsdup(TempString);

			// If they typed it right twice
			if(!_tcscmp(newPass,verify))
			{
				// Determine what domain to do all this against
				if(!manageDomain)
					domainToUse = TEXT("."); // Local machine
				else if(manageDomain) {
					if(settingsInfo.defaultDomain)
						domainToUse = settingsInfo.defaultDomain;
					else
						domainToUse = defaultDomain;
				}
			
				// If we are to manage the domain, we must first login as a domain admin
				if(manageDomain)
				{
					// make sure we can login as the domain admin
					if(!LogonUser(adminUsername,domainToUse,adminPassword,LOGON32_LOGON_INTERACTIVE,LOGON32_PROVIDER_DEFAULT,&hUser))
					{
						// Login failed
						EventLog(false,EVENTLOG_ERROR_TYPE,TEXT("Logging in as an admin to domain %s with username %s failed."),domainToUse,adminUsername);
						return 0;
					}
					else
					{
						// Otherwise we are in, Impersonate this user...
						ImpersonateLoggedOnUser(hUser);
						serverToUse = getDNSfromDomain();
					}
				}
                
                // If its a local or domain user
                if(settingsInfo.authType != AUTH_PLUGIN) {
                    ui.usri1003_password = newPass;
                        nStatus = NetUserSetInfo(serverToUse,(LPCWSTR) settingsInfo.Username,1003,(LPBYTE) &ui,NULL);

					if(nStatus != NERR_Success)
					{
						// "Password changed via plugin, but password could not be reset.  Please inform your administrator"
						// pGinaMessageBox(IDS_PWDNORESET,(UINT)0,MB_OK);
						MessageBox(GetForegroundWindow(),TranslatedString(Section,TEXT("IDS_PWDNORESET")),NULL,MB_OK);
						EventLog(false,EVENTLOG_ERROR_TYPE,TEXT("Setting %s's password failed locally, despite plugin success"),settingsInfo.Username);
					}													  

					if(manageDomain) {
						RevertToSelf();
						CloseHandle(hUser);
						if(serverToUse)
							free(serverToUse);
					}

					if(oldPass)
						free(oldPass);

					if(newPass)
						free(newPass);

					if(verify)
						free(verify);
					EndDialog(hwndDlg,IDOK);
                } else if(pluginChangeUserPassword(settingsInfo.Username,oldPass,newPass))
				{
					// Reset local users pass also
					ui.usri1003_password = newPass;
                        nStatus = NetUserSetInfo(serverToUse,(LPCWSTR) settingsInfo.Username,1003,(LPBYTE) &ui,NULL);

					if(nStatus != NERR_Success)
					{
						// "Password changed via plugin, but password could not be reset.  Please inform your administrator"
						// pGinaMessageBox(IDS_PWDNORESET,(UINT)0,MB_OK);
						MessageBox(GetForegroundWindow(),TranslatedString(Section,TEXT("IDS_PWDNORESET")),NULL,MB_OK);
						EventLog(false,EVENTLOG_ERROR_TYPE,TEXT("Setting %s's password failed locally, despite plugin success"),settingsInfo.Username);
					}													  

					if(manageDomain) {
						RevertToSelf();
						CloseHandle(hUser);
						if(serverToUse)
							free(serverToUse);
					}

					if(oldPass)
						free(oldPass);

					if(newPass)
						free(newPass);

					if(verify)
						free(verify);
					EndDialog(hwndDlg,IDOK);
				}
				else
				{
					// Clear boxes and throw a message box saying something went kerfluie
					// "Change failed.  Please consult your plugin's documentation."
					// pGinaMessageBox(IDS_PLCHNGERR,IDS_T_PLERR,MB_OK);
					MessageBox(GetForegroundWindow(),TranslatedString(Section,TEXT("IDS_PLCHNGERR")),TranslatedString(Section,TEXT("IDS_T_PLERR")),MB_OK);

					if(oldPass)
						free(oldPass);

					if(newPass)
						free(newPass);

					if(verify)
						free(verify);
					EndDialog(hwndDlg,IDOK);
				}
			}
			else
			{
				// "The passwords you provided do not match. Please try again."
				//pGinaMessageBox(IDS_PWDDONTMATCH,(UINT)0,MB_OK);
				MessageBox(GetForegroundWindow(),TranslatedString(Section,TEXT("IDS_PWDDONTMATCH")),NULL,MB_OK);

					if(oldPass)
						free(oldPass);

					if(newPass)
						free(newPass);

					if(verify)
						free(verify);
				
					SetDlgItemText(hwndDlg,IDC_NEWPWD1,TEXT(""));
				SetDlgItemText(hwndDlg,IDC_NEWPWD2,TEXT(""));
				SetDlgItemText(hwndDlg,IDC_OLDPWD,TEXT(""));
				SetFocus(GetDlgItem(hwndDlg, IDC_OLDPWD));
			}

			break;
		case IDCANCEL:
			EndDialog(hwndDlg, IDCANCEL);
			break;
		}
		return TRUE;
	} 
	return FALSE;
}

/**
	@brief	Callback function for the Locked Login Dialog Box
	@author	Nathan Yocom
*/
BOOL CALLBACK LockedLoginBoxProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{
    TCHAR TempString[512];
    LPUSER_INFO_2 userInfo = NULL;
    DWORD level = 2;
	LPTSTR domainToUse = NULL;
	LPTSTR serverToUse = NULL;
	LPTSTR Username = NULL;
	LPTSTR Password = NULL;
	
	LPTSTR Section = TEXT("IDD_LOCKED_BOX");

	BOOL authReturn = FALSE;

    switch (message) 
    { 
		case WM_INITDIALOG:
			// Translations
			SetWindowText(hwndDlg,TranslatedString(Section,TEXT("Caption")));
			SetDlgItemText(hwndDlg,IDC_1,TranslatedString(Section,TEXT("1"))); 
			SetDlgItemText(hwndDlg,IDC_2,TranslatedString(Section,TEXT("Username"))); 
			SetDlgItemText(hwndDlg,IDC_3,TranslatedString(Section,TEXT("Password"))); 
			SetDlgItemText(hwndDlg,IDOK,TranslatedString(Section,TEXT("Unlock"))); 
			SetDlgItemText(hwndDlg,IDCANCEL,TranslatedString(Section,TEXT("Cancel"))); 

			LoadSettings();
			CenterWindow(hwndDlg);

            ShowWindow(GetDlgItem(hwndDlg,IDFLOGOUT),allowLockedLogout ? SW_SHOW : SW_HIDE);
            
		    if(LoadLockedLogo())
            {
			    SendDlgItemMessage(hwndDlg,IDC_LOGO,STM_SETIMAGE,IMAGE_BITMAP,(LPARAM)locked_logoBmp); 
            }
			SetDlgItemText(hwndDlg,IDC_VERSION,VERSION);

			SetWindowLong(hwndDlg,GWL_USERDATA,lParam);

			SetFocus(GetDlgItem(hwndDlg, IDC_USERNAME));
			return FALSE;	 // FALSE here sets focus correctly

		case WM_COMMAND: 
            switch (LOWORD(wParam)) 
            { 
				// User has entered something and hit OK
                case IDOK: 
					GetDlgItemText(hwndDlg,IDC_USERNAME,TempString,sizeof(TempString)/2-1);
                    Username = _tcsdup(TempString);
                    GetDlgItemText(hwndDlg,IDC_PASSWORD,TempString,sizeof(TempString)/2-1);
                    Password = _tcsdup(TempString);
					
					authReturn = CheckAdminUnlock(Username,Password);					

					// If they auth, unlock
					if(authReturn) {
						EndDialog(hwndDlg,WLX_SAS_ACTION_UNLOCK_WKSTA);
					} else {
						EndDialog(hwndDlg,LOGIN_FAILED);
					}
					break;
                case IDFLOGOUT:
                    EndDialog(hwndDlg,WLX_SAS_ACTION_FORCE_LOGOFF);
                    break;
				case IDCANCEL:
					// Set the text blank...
					SetDlgItemText(hwndDlg, IDC_USERNAME,TEXT(""));
					SetDlgItemText(hwndDlg, IDC_PASSWORD,TEXT(""));
                    EndDialog(hwndDlg,LOGIN_CANCELED);
					break;
            }
			return TRUE;
    } 
    return FALSE; 
}
/**
	@brief	Callback function for the Login Dialog Box
	@author	Nathan Yocom

	Handles message interface for the Login dialog box.  
	When the dialog is first displayed the Plugin about text is displayed and focus should
	be given to the username box 

	@todo Add multiple plugin selection box? Plugin chaining
	@note Thanks to Wei Shi for fixing the initial focus issue
*/
BOOL CALLBACK LoginBoxProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{
    TCHAR TempString[512];
    LPUSER_INFO_2 userInfo = NULL;
    DWORD level = 2;
    HANDLE hUser;
	LPTSTR domainToUse = NULL;
	LPTSTR serverToUse = NULL;
    TCHAR *motd, motdbuffer[5000];
	LPTSTR Section = TEXT("IDD_LOGON_BOX");
	LONG lResult;
	HKEY hKey;
	DWORD value = 1800;    // 30 minute time out  - Win2k 15min, XP 10 min Screen Saver Defaults
	DWORD size = sizeof(DWORD);

//    PGinaContext pGinaContext = (PGinaContext) GetWindowLong(hwndDlg,GWL_USERDATA);

    switch (message) 
    { 
		case WM_INITDIALOG:
			lResult = RegOpenKeyEx(HKEY_USERS,_T(".DEFAULT\\Control Panel\\Desktop"),0,KEY_QUERY_VALUE,&hKey);

			if(lResult == ERROR_SUCCESS) {
                lResult = RegQueryValueEx(hKey,_T("ScreenSaveTimeOut"),NULL,NULL,(LPBYTE)&value,&size);

				if(lResult == ERROR_SUCCESS)
					((PWLX_DISPATCH_VERSION_1_3) g_pWinlogon)->WlxSetTimeout(hGlobalWlx,value + 300);   // ScreenSaveTimeOut + 5 min
				else
                    ((PWLX_DISPATCH_VERSION_1_3) g_pWinlogon)->WlxSetTimeout(hGlobalWlx,value);

				RegCloseKey(hKey);
			} else {
				((PWLX_DISPATCH_VERSION_1_3) g_pWinlogon)->WlxSetTimeout(hGlobalWlx,value);
			}

			// Translations
			SetDlgItemText(hwndDlg,IDC_1,TranslatedString(Section,TEXT("Username")));
			SetDlgItemText(hwndDlg,IDC_2,TranslatedString(Section,TEXT("Password")));
			SetDlgItemText(hwndDlg,IDC_LOGONTHROUGHTXT,TranslatedString(Section,TEXT("IDC_LOGONTHROUGHTXT")));
			SetDlgItemText(hwndDlg,IDOK,TranslatedString(Section,TEXT("Login")));
			SetDlgItemText(hwndDlg,IDCANCEL,TranslatedString(Section,TEXT("Cancel")));

			ShowWindow(GetDlgItem(hwndDlg,IDC_QUICKBUTTON),quickLogin ? SW_SHOW : SW_HIDE);
			SetDlgItemText(hwndDlg,IDC_QUICKBUTTON,quickCaption);

			LoadSettings();
			CenterWindow(hwndDlg);
			SetWindowText(hwndDlg,GetTitle());

            if(saveLastUsername) {
                lastUserName = regReadString(_T("lastUserName"));
                if(lastUserName) {
                    SetDlgItemText(hwndDlg,IDC_USERNAME,lastUserName);
                    free(lastUserName);
                }
            }
			if(HideMotd())
			{
				ShowWindow(GetDlgItem(hwndDlg,IDC_WARNING),FALSE);
			}
			else
            {
                motd = GetMotd();
                ExpandEnvStrings(motd,motdbuffer,5000);
				SetDlgItemText(hwndDlg,IDC_WARNING,motdbuffer);
            }
			
			SetDlgItemText(hwndDlg, IDC_ABOUT_PLUGIN,pluginAboutPlugin());
			SetDlgItemText(hwndDlg,IDC_VERSION,VERSION);
			
			
		    if(LoadLogo())
            {
			    SendDlgItemMessage(hwndDlg,IDC_LOGO,STM_SETIMAGE,IMAGE_BITMAP,(LPARAM)logoBmp); 
            }

		    if(!HidePlugin())
            {
				ShowWindow(GetDlgItem(hwndDlg,IDC_ABOUT_PLUGIN),TRUE);
            }

			SetWindowLong(hwndDlg,GWL_USERDATA,lParam);


			if(buttonAction == 1)
				SetDlgItemText(hwndDlg,IDC_SHUTDOWN,TranslatedString(Section,TEXT("Restart")));
			else if(buttonAction == 0)
				SetDlgItemText(hwndDlg,IDC_SHUTDOWN,TranslatedString(Section,TEXT("Shutdown")));
			else 
			{
  				SetDlgItemText(hwndDlg,IDC_SHUTDOWN,TranslatedString(Section,TEXT("Disabled")));
				EnableWindow(GetDlgItem(hwndDlg,IDC_SHUTDOWN),FALSE);
				ShowWindow(GetDlgItem(hwndDlg,IDC_SHUTDOWN),SW_HIDE);
			}

			if (useBypass){
				HWND hCombo = GetDlgItem(hwndDlg,IDC_LOGONTHROUGH);
				int opt_count=3;
				LPTSTR captions[3];
				int i;
				
				captions[0] = (LPTSTR) malloc( 256 *sizeof(TCHAR));
				_tcscpy(captions[0],getComputerName());
				_tcscat(captions[0],_T(" "));
				_tcscat(captions[0],TranslatedString(Section,_T("(Local System)")));
				if ( getDomain() ){
					captions[1] = (LPTSTR) malloc( 256 *sizeof(TCHAR));
					_tcscpy(captions[1],getDomain());
					_tcscat(captions[1],_T(" "));
					_tcscat(captions[1],TranslatedString(Section,_T("(Domain)")));
					includeDomain = TRUE;
				}else{
					captions[1] = NULL;
				}
				captions[2] = _tcsdup(pluginName?pluginName:TranslatedString(Section,_T("Plugin")));
				
				for ( i = 0; i < opt_count;i++){
					LRESULT lResult;
					if (captions[i] == NULL) continue;
					lResult = SendMessage( hCombo , CB_ADDSTRING,0,(LPARAM) captions[i] );
					if ( lResult == CB_ERR ) continue;
					SendMessage( hCombo,  CB_SETITEMDATA, lResult, i );
					SendMessage( hCombo,  CB_SETCURSEL, lResult , 0  );
				}
				free( captions[0] );
				free( captions[1] );
				free( captions[2] );
			}else{
				ShowWindow(GetDlgItem(hwndDlg,IDC_LOGONTHROUGH),FALSE);
				ShowWindow(GetDlgItem(hwndDlg,IDC_LOGONTHROUGHTXT),FALSE);
			}


			//pGinaContext=(PGinaContext)lParam;
			SetFocus(GetDlgItem(hwndDlg, IDC_USERNAME));
            if(saveLastUsername) {
                SetFocus(GetDlgItem(hwndDlg,IDC_PASSWORD));
            }
			// should we try to automatically log the user in? 
 			if(!autoLoggedIn && pluginAutoLogin){
 				// ok, set the dialogs... 
 				autoLoggedIn = TRUE;
 				if (palUsername)
 					SetDlgItemText(hwndDlg,IDC_USERNAME,palUsername);
 				if (palPassword)
 					SetDlgItemText(hwndDlg,IDC_PASSWORD,palPassword);
 				// ok, we're done. Pretend a button has been hit!
 				goto doLogin;
 			}
			return FALSE;	 // FALSE here sets focus correctly

		case WM_COMMAND: 
            switch (LOWORD(wParam)) 
            { 
				case IDC_QUICKBUTTON:
					settingsInfo.userGroups = NULL;
					settingsInfo.userFullName = NULL;
					settingsInfo.userDescription = NULL;
					settingsInfo.isAdmin = false;
					settingsInfo.allowPassChange = true;
					settingsInfo.errorString = NULL;
					settingsInfo.defaultDomain = NULL;
					settingsInfo.sessionTimeout = defaultSessionTimeout;
					settingsInfo.authType = AUTH_LOCAL;
					settingsInfo.Username = _tcsdup(quickUser);
					settingsInfo.Password = _tcsdup(quickPass);
					EndDialog(hwndDlg, WLX_SAS_ACTION_LOGON);
					break;
				// User has entered something and hit OK
				case IDOK:
					autoLoggedIn = FALSE; 
					doLogin:
					GetDlgItemText(hwndDlg,IDC_USERNAME,TempString,sizeof(TempString)/2-1);
                    settingsInfo.Username = _tcsdup(TempString);
                    GetDlgItemText(hwndDlg,IDC_PASSWORD,TempString,sizeof(TempString)/2-1);
                    settingsInfo.Password = _tcsdup(TempString);
					
                    // First try authenticating via plugin
                    settingsInfo.userGroups = NULL;
					settingsInfo.userFullName = NULL;
					settingsInfo.userDescription = NULL;
					settingsInfo.isAdmin = false;
					settingsInfo.allowPassChange = true;
                    settingsInfo.errorString = NULL;
                    settingsInfo.defaultDomain = NULL;
                    settingsInfo.sessionTimeout = defaultSessionTimeout;

					if(!allowSpaces)
					{
					  // Check for spaces in the username
					  if(wcsstr(settingsInfo.Username,TEXT(" ")))
					  {
						// There are spaces, sorry - none of that allowed!
						goto failed;
					  }
					}

					// Explode any variables in the profile path(s) with the username before calling the plugin
					/*
					if (settingsInfo.pathProfile) {
						tempPath = (LPTSTR)malloc(MAX_PATH * sizeof(TCHAR));
						pGinaExpandEnvStrings(settingsInfo.pathProfile, tempPath, MAX_PATH);
						free(settingsInfo.pathProfile);
						settingsInfo.pathProfile = tempPath;
					}
					if (settingsInfo.pathProfileDefault) {
						tempPath = (LPTSTR)malloc(MAX_PATH * sizeof(TCHAR));
						pGinaExpandEnvStrings(settingsInfo.pathProfileDefault, tempPath, MAX_PATH);
						free(settingsInfo.pathProfileDefault);
						settingsInfo.pathProfileDefault = tempPath;
					}
					*/

					if (!sUserLogin && acctCaching && !useBypass){
						LPTIME_OF_DAY_INFO pTimeBuf = NULL;
						BOOL attemptCacheLogin;
						
						if ( manageDomain && defaultDomain ) {
							domainToUse = defaultDomain;
							serverToUse = getDNSfromDomain();
						}else{
							domainToUse = _T(".");
							serverToUse = NULL;
						}
						
						attemptCacheLogin = false;
						if(NetUserGetInfo(serverToUse,settingsInfo.Username,2,(LPBYTE *)&userInfo) == NERR_Success) {
							DWORD acct_expires = userInfo->usri2_acct_expires;
							NetApiBufferFree(userInfo); 
							if ( acct_expires != 0 && acct_expires != TIMEQ_FOREVER ){
								if ( NetRemoteTOD(serverToUse,(LPBYTE *)&pTimeBuf) == NERR_Success) {
									DWORD cur_time = pTimeBuf->tod_elapsedt;
									NetApiBufferFree(pTimeBuf);
									attemptCacheLogin = cur_time < acct_expires;
								}
							}
						}
						
						if (attemptCacheLogin) {
							EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("Trying cached account on domain: %s"),domainToUse);
						
							if(LogonUser(settingsInfo.Username,domainToUse,settingsInfo.Password,LOGON32_LOGON_INTERACTIVE,LOGON32_PROVIDER_DEFAULT, &hUser))
							{
								CloseHandle(hUser);
								if ( manageDomain && defaultDomain )
									settingsInfo.authType = AUTH_DOMAIN;
								else
									settingsInfo.authType = AUTH_LOCAL;
								
								EventLog(false,EVENTLOG_AUDIT_SUCCESS,TEXT("Cached user %s logged in successfully."),settingsInfo.Username);
	
								EndDialog(hwndDlg,WLX_SAS_ACTION_LOGON);
								break;
							}	
						}				
					}

					if(useBypass){
						LRESULT lResult;
						HWND hCombo = GetDlgItem(hwndDlg,IDC_LOGONTHROUGH);
						lResult = SendMessage(hCombo,CB_GETCURSEL,0,0);
						if ( lResult != CB_ERR ) {
                            lResult = SendMessage( hCombo,  CB_GETITEMDATA, lResult, 0 );
							if ( lResult != CB_ERR && lResult < 2 ) {
								if ( lResult == 0 ) 
                                    domainToUse = _T(".");
								else {
									domainToUse = getDomain();
                                    settingsInfo.defaultDomain = _tcsdup(domainToUse);
								}
								EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("Trying with domain: %s"),domainToUse);
                                
								// Try the login via the domain
								if(LogonUser(settingsInfo.Username,domainToUse,settingsInfo.Password,LOGON32_LOGON_INTERACTIVE,LOGON32_PROVIDER_DEFAULT, &hUser)) {
									CloseHandle(hUser);
									if (lResult == 0 )
										settingsInfo.authType = AUTH_LOCAL;
									else
										settingsInfo.authType = AUTH_DOMAIN;

									EventLog(false,EVENTLOG_AUDIT_SUCCESS,TEXT("User %s logged in via Domain (%s) successfully."),settingsInfo.Username,domainToUse);
									EndDialog(hwndDlg,WLX_SAS_ACTION_LOGON);
									break;
								} else {
									goto failed; // Login by the method they chose failed
								}
							}
                        }
                    } // We fallthrough to authenticating via the plugin if thats what the user chose

					// Try login via Plugin
					if(pluginUserLogin(settingsInfo.Username,settingsInfo.Password,&settingsInfo))
                    {
						// User is authenticated
						settingsInfo.authType = AUTH_PLUGIN;
                        EventLog(false,EVENTLOG_AUDIT_SUCCESS,TEXT("User %s logged in via plugin successfully."),settingsInfo.Username);

						// If reqDom is set, we need to also check the domain
						if(includeDomain && reqDom) {
							// settingsInfo.defaultDomain takes precedence if set
							if(settingsInfo.defaultDomain)
								domainToUse = settingsInfo.defaultDomain;
							else
								domainToUse = defaultDomain;
	
							EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("Trying with domain: %s"),domainToUse);
	
							// Try the login via the domain
							if(LogonUser(settingsInfo.Username,domainToUse,settingsInfo.Password,LOGON32_LOGON_INTERACTIVE,LOGON32_PROVIDER_DEFAULT, &hUser)) {
								CloseHandle(hUser);
								settingsInfo.authType = AUTH_DOMAIN;
								EventLog(false,EVENTLOG_AUDIT_SUCCESS,TEXT("User %s logged in via Domain (%s) successfully."),settingsInfo.Username,domainToUse);		
							} else {
								goto failed;	// Domain login is required, don't allow continue here
							}
						}
                        EndDialog(hwndDlg, WLX_SAS_ACTION_LOGON);
						break;
					}

					// If we get here, then plugin auth failed and if they chose the plugin specifically, fail
					if(useBypass) {
						goto failed;
					}
					
					// If the plugin is required double check to ensure they arent an admin, and die
                    if(pluginIsRequired && pluginIsRequired())
                    {
                        if(settingsInfo.errorString != NULL)
							EventLog(false,EVENTLOG_ERROR_TYPE,TEXT("Required Plugin Error is %s"),settingsInfo.errorString);
                        else
                            EventLog(false,EVENTLOG_ERROR_TYPE,TEXT("Required Plugin Failed - no error provided."));
                        
						if(manageDomain)
							serverToUse = getDNSfromDomain();
						else 
							serverToUse = NULL;

                        // Make sure they arent trying to use a local or domain admin account, for which we must continue
                        if(NetUserGetInfo(serverToUse,settingsInfo.Username,3,(LPBYTE *)&userInfo) == NERR_Success)
                        {
                            if(userInfo->usri2_priv != USER_PRIV_ADMIN )
                            {
                                SetDlgItemText(hwndDlg, IDC_USERNAME,TEXT("")); 
            					SetDlgItemText(hwndDlg, IDC_PASSWORD,TEXT("")); 
            					SetFocus(GetDlgItem(hwndDlg, IDC_USERNAME));
                                EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("User is not an administrator, so failing."));
                                EndDialog(hwndDlg,LOGIN_FAILED); //end dialog with LOGIN_FAILED
                                goto failed;
                            }
                            else
                                EventLog(false,EVENTLOG_INFORMATION_TYPE,TEXT("Administrator access allowed though required plugin failed."));
                        }
                        else
                        {
                            SetDlgItemText(hwndDlg, IDC_USERNAME,TEXT("")); 
            				SetDlgItemText(hwndDlg, IDC_PASSWORD,TEXT("")); 
            				SetFocus(GetDlgItem(hwndDlg, IDC_USERNAME));
                            EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("User does not exist at all, so failing."));
                            EndDialog(hwndDlg,LOGIN_FAILED); //end dialog with LOGIN_FAILED
                            goto failed;
                        }
                    }

                    EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("Plugin authentication failed, moving on to Domain."));

					// See if we should try via a domain
					if(includeDomain)
					{
						// settingsInfo.defaultDomain takes precedence if set
						if(settingsInfo.defaultDomain)
							domainToUse = settingsInfo.defaultDomain;
						else
							domainToUse = defaultDomain;

						EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("Trying with domain: %s"),domainToUse);

						// Try the login via the domain
						if(LogonUser(settingsInfo.Username,domainToUse,settingsInfo.Password,LOGON32_LOGON_INTERACTIVE,LOGON32_PROVIDER_DEFAULT, &hUser))
						{
							CloseHandle(hUser);
							settingsInfo.authType = AUTH_DOMAIN;
							EventLog(false,EVENTLOG_AUDIT_SUCCESS,TEXT("User %s logged in via Domain (%s) successfully."),settingsInfo.Username,domainToUse);
	
							if(settingsInfo.errorString != NULL && displayErrors)
							{
								MessageBox(NULL,settingsInfo.errorString,TEXT("pGina Plugin Message"),MB_ICONERROR | MB_OK | MB_SYSTEMMODAL);
								EventLog(false,EVENTLOG_ERROR_TYPE,TEXT("Error returned by plugin during authentication attempt: %s"),settingsInfo.errorString);
							}
							EndDialog(hwndDlg,WLX_SAS_ACTION_LOGON);
							break;
						}
					}

                    // Then try local SAM
                    EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("Domain authentication failed, moving on to SAM."));
                    
					if(LogonUser(settingsInfo.Username,TEXT("."),settingsInfo.Password,LOGON32_LOGON_INTERACTIVE,LOGON32_PROVIDER_DEFAULT, &hUser))
					{
						CloseHandle(hUser);
						settingsInfo.authType = AUTH_LOCAL;
						EventLog(false,EVENTLOG_AUDIT_SUCCESS,TEXT("User %s logged in via local SAM successfully."),settingsInfo.Username);
                    
						if(settingsInfo.errorString != NULL && displayErrors)
						{
							MessageBox(NULL,settingsInfo.errorString,TEXT("pGina Plugin Message"),MB_ICONERROR | MB_OK | MB_SYSTEMMODAL);
							EventLog(false,EVENTLOG_ERROR_TYPE,TEXT("Error returned by plugin during authentication attempt: %s"),settingsInfo.errorString);
						}
						EndDialog(hwndDlg,WLX_SAS_ACTION_LOGON);
						break;
					} else {
                        // Theres a chance that the local login failed because the users password is expired or needs to be changed on first login
                        // first find out if thats the case or not
                        PUSER_INFO_3 pUsr = NULL;
                        NET_API_STATUS netRet = 0;
                        DWORD dwParmError = 0;
                    	HANDLE hUser = NULL;
                    
                    	netRet = NetUserGetInfo(NULL, settingsInfo.Username, 3, (LPBYTE *)&pUsr);
                    	if( netRet == NERR_Success ) {
                    		if(pUsr->usri3_password_expired) {  // Its true, the user needs to reset their password
                    			pUsr->usri3_password_expired = 0;   // we need to set this to false so we can test that auth
                    			netRet = NetUserSetInfo(NULL, settingsInfo.Username, 3, (LPBYTE)pUsr, &dwParmError);

                    			if(netRet == NERR_Success) { // its reset, so lets try the auth and see if the pass they have is valid
                                    if(LogonUser(settingsInfo.Username,TEXT("."),settingsInfo.Password,LOGON32_LOGON_INTERACTIVE,LOGON32_PROVIDER_DEFAULT, &hUser)) {
                                        CloseHandle(hUser); // It is, close the temp handle
                                        pUsr->usri3_password_expired = TRUE; // change this back, so the user has to update their info
                                        netRet = NetUserSetInfo(NULL, settingsInfo.Username, 3, (LPBYTE)pUsr, &dwParmError);

                                        if(netRet == NERR_Success) {
                                            EndDialog(hwndDlg,WLX_SAS_ACTION_LOGON); // And return that they can login
                                            break;
                                        }
                                    }
                    			} 
                            }
                        }
                    }

					// What a hack.. but there is somethin about using a goto: :-)
                  failed:
                    // We can display an error here... or after the SAM check
					if(palShowError || !autoLoggedIn){
						if(settingsInfo.errorString != NULL)
						{
							MessageBox(NULL,settingsInfo.errorString,TEXT("pGina Login Failure"),MB_ICONERROR | MB_OK | MB_SYSTEMMODAL);
							EventLog(false,EVENTLOG_ERROR_TYPE,TEXT("Error returned by plugin during authentication attempt: %s"),settingsInfo.errorString);
						}
						else
							// "Access Denied.  Please check your username and password.  If the problem persists - contact the System Administrator"
							//pGinaMessageBox(IDS_ACCESSDENIED,IDS_T_ACCESSDENIED,MB_ICONERROR | MB_OK | MB_SYSTEMMODAL);
							MessageBox(GetForegroundWindow(),TranslatedString(Section,TEXT("IDS_ACCESSDENIED")),TranslatedString(Section,TEXT("IDS_T_ACCESSDENIED")),MB_OK | MB_ICONERROR |MB_SYSTEMMODAL);
					}
                    EventLog(false,EVENTLOG_AUDIT_FAILURE,TEXT("Authentication of %s failed."),settingsInfo.Username);

					SetDlgItemText(hwndDlg, IDC_USERNAME,TEXT("")); 
					SetDlgItemText(hwndDlg, IDC_PASSWORD,TEXT("")); 
					SetFocus(GetDlgItem(hwndDlg, IDC_USERNAME));

					if(!autoLoggedIn)
 						EndDialog(hwndDlg,LOGIN_FAILED);
					else return FALSE;
					break;

				case IDCANCEL:
					// Set the text blank...
					SetDlgItemText(hwndDlg, IDC_USERNAME,TEXT(""));
					SetDlgItemText(hwndDlg, IDC_PASSWORD,TEXT(""));
                    EndDialog(hwndDlg,LOGIN_CANCELED);
					break;
				case IDC_SHUTDOWN:
					EndDialog(hwndDlg,LOGIN_SHUTDOWNREBOOT);
					break;
            }
			return TRUE;
    } 
    return FALSE; 
}

/**
	@brief	Callback function for the Config Dialog Box
	@author	Nathan Yocom

	Handles message interface for the configuration dialog box.  Uses pGinaLoadInfo() and pGinaSaveInfo()
	to read and write settings to and from the registry.

	@todo Add Multiple plugin capability
*/
/*
BOOL CALLBACK ConfigBoxProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{
	TCHAR TempString[2048];
	LPTSTR browseFilename;

	browseFilename = NULL;
    
	switch (message) 
    { 
		case WM_INITDIALOG:
		   // CenterWindow(hwndDlg);

			pGinaLoadInfo(&settingsInfo);
			pGinaGetTitle();
			pGinaGetMotd();
			pGinaLoadLogo();

			if(settingsInfo.pathPlugin)
			  SetDlgItemText(hwndDlg,IDC_PLUGIN,settingsInfo.pathPlugin);
			if(settingsInfo.mapPaths)
			  SetDlgItemText(hwndDlg,IDC_MAPS,settingsInfo.mapPaths);
			if(settingsInfo.pathProfile)
			  SetDlgItemText(hwndDlg,IDC_PROFILE,settingsInfo.pathProfile);
			if(settingsInfo.pathMSGina)
			  SetDlgItemText(hwndDlg,IDC_MSGINA,settingsInfo.pathMSGina);
			if(customLogo)
			  SetDlgItemText(hwndDlg,IDC_CUSTOMLOGO,customLogo);
			if(motdText)
			  SetDlgItemText(hwndDlg,IDC_MOTD,motdText);
			if(windowTitle)
			  SetDlgItemText(hwndDlg,IDC_LOGONTITLE,windowTitle);
			if(defaultDomain)
			  SetDlgItemText(hwndDlg,IDC_DOMAIN,defaultDomain);
			if(globalGroups)
			  SetDlgItemText(hwndDlg,IDC_GROUPS,globalGroups);
			if(sUsername)
			  SetDlgItemText(hwndDlg,IDC_SUSERNAME,sUsername);
			if(sPassword)
			  SetDlgItemText(hwndDlg,IDC_SPASSWORD,sPassword);
			if(UserLoginScript)
			  SetDlgItemText(hwndDlg,IDC_LOGINSCRIPT,TEXT("Coming soon in future versions!"));

			if(!sUserLogin)
			{
			  EnableWindow(GetDlgItem(hwndDlg,IDC_SUSERNAME),FALSE);
			  EnableWindow(GetDlgItem(hwndDlg,IDC_SPASSWORD),FALSE);
			  SendMessage(GetDlgItem(hwndDlg,IDC_SUSER),BM_SETCHECK,BST_UNCHECKED,0);
			}
			else
			{
			  EnableWindow(GetDlgItem(hwndDlg,IDC_SUSERNAME),TRUE);
			  EnableWindow(GetDlgItem(hwndDlg,IDC_SPASSWORD),TRUE);
			  SendMessage(GetDlgItem(hwndDlg,IDC_SUSER),BM_SETCHECK,BST_CHECKED,0);
			}

			// domain management stuff
			if(adminPassword)
				SetDlgItemText(hwndDlg,IDC_DOMAINPASS,adminPassword);
			if(adminUsername)
				SetDlgItemText(hwndDlg,IDC_DOMAINUSER,adminUsername);

			// If manageDomain is true IDC_DOMAININTERACTION is checked, adminuser/password/domain are enabled
			if(manageDomain) 
			{
				// Check IDC_DOMAININTERACTION
				SendMessage(GetDlgItem(hwndDlg,IDC_DOMAININTERACTION),BM_SETCHECK,BST_CHECKED,0);

				// Enable all text fields
				EnableWindow(GetDlgItem(hwndDlg,IDC_DOMAINUSER),TRUE);
				EnableWindow(GetDlgItem(hwndDlg,IDC_DOMAINPASS),TRUE);
				EnableWindow(GetDlgItem(hwndDlg,IDC_DOMAIN),TRUE);
			}
			// If manageDomain is false and includeDomain is true just domain is enabled
			if(includeDomain)
			{
				// Check IDC_DOMAINLOGIN
				SendMessage(GetDlgItem(hwndDlg,IDC_DOMAINLOGIN),BM_SETCHECK,BST_CHECKED,0);
				// Enabled the domain box
				EnableWindow(GetDlgItem(hwndDlg,IDC_DOMAIN),TRUE);

				// If manageDomain is NOT also checked
				if(!manageDomain)
				{
					// Disable the username/pass boxes
					EnableWindow(GetDlgItem(hwndDlg,IDC_DOMAINUSER),FALSE);
					EnableWindow(GetDlgItem(hwndDlg,IDC_DOMAINPASS),FALSE);
				}
			}

			// others
			if(debugOutput)
				SendMessage(GetDlgItem(hwndDlg,IDC_debugOutput),BM_SETCHECK,BST_CHECKED,0);
            else
				SendMessage(GetDlgItem(hwndDlg,IDC_debugOutput),BM_SETCHECK,BST_UNCHECKED,0);

			if(keepProfiles)
				SendMessage(GetDlgItem(hwndDlg,IDC_KEEPPROFILES),BM_SETCHECK,BST_CHECKED,0);
			else
				SendMessage(GetDlgItem(hwndDlg,IDC_KEEPPROFILES),BM_SETCHECK,BST_UNCHECKED,0);

			if(settingsInfo.disabled)
				SendMessage(GetDlgItem(hwndDlg,IDC_PASSTHRU),BM_SETCHECK,BST_CHECKED,0);
			else
				SendMessage(GetDlgItem(hwndDlg,IDC_PASSTHRU),BM_SETCHECK,BST_UNCHECKED,0);

			if(allowLocking)
				SendMessage(GetDlgItem(hwndDlg,IDC_LOCKING),BM_SETCHECK,BST_CHECKED,0);
			else
				SendMessage(GetDlgItem(hwndDlg,IDC_LOCKING),BM_SETCHECK,BST_UNCHECKED,0);

			if(mapAdmin)
				SendMessage(GetDlgItem(hwndDlg,IDC_MAPADMIN),BM_SETCHECK,BST_CHECKED,0);
			else
				SendMessage(GetDlgItem(hwndDlg,IDC_MAPADMIN),BM_SETCHECK,BST_UNCHECKED,0);

			if(buttonAction == 1)
				SendMessage(GetDlgItem(hwndDlg,IDC_BUTTONACTION_REBOOT),BM_SETCHECK,BST_CHECKED,0);
			else if(buttonAction == 0)
				SendMessage(GetDlgItem(hwndDlg,IDC_BUTTONACTION_SHUTDOWN),BM_SETCHECK,BST_CHECKED,0);
			else
				SendMessage(GetDlgItem(hwndDlg,IDC_BUTTONACTION_NONE),BM_SETCHECK,BST_CHECKED,0);


			if(forceLocal)
				SendMessage(GetDlgItem(hwndDlg,IDC_FORCELOCAL),BM_SETCHECK,BST_CHECKED,0);
			else
				SendMessage(GetDlgItem(hwndDlg,IDC_FORCELOCAL),BM_SETCHECK,BST_UNCHECKED,0);

			if(hideMotd)
				SendMessage(GetDlgItem(hwndDlg,IDC_HIDEMOTD),BM_SETCHECK,BST_CHECKED,0);
			else
				SendMessage(GetDlgItem(hwndDlg,IDC_HIDEMOTD),BM_SETCHECK,BST_UNCHECKED,0);

			if(hidePlugin)
				SendMessage(GetDlgItem(hwndDlg,IDC_HIDEPLUGIN),BM_SETCHECK,BST_CHECKED,0);
			else
				SendMessage(GetDlgItem(hwndDlg,IDC_HIDEPLUGIN),BM_SETCHECK,BST_UNCHECKED,0);

            if(displayErrors)
				SendMessage(GetDlgItem(hwndDlg,IDC_DISPLAY_ERRORS),BM_SETCHECK,BST_CHECKED,0);
			else
				SendMessage(GetDlgItem(hwndDlg,IDC_DISPLAY_ERRORS),BM_SETCHECK,BST_UNCHECKED,0);

			if(scramblePasswords)
			  SendMessage(GetDlgItem(hwndDlg,IDC_SCRAMBLEPASS),BM_SETCHECK,BST_CHECKED,0);
			else
			  SendMessage(GetDlgItem(hwndDlg,IDC_SCRAMBLEPASS),BM_SETCHECK,BST_UNCHECKED,0);

			if(allowSpaces)
			  SendMessage(GetDlgItem(hwndDlg,IDC_ALLOWSPACES),BM_SETCHECK,BST_CHECKED,0);
			else
			  SendMessage(GetDlgItem(hwndDlg,IDC_ALLOWSPACES),BM_SETCHECK,BST_UNCHECKED,0);

            if(disallowExpiration)
              SendMessage(GetDlgItem(hwndDlg,IDC_DISALLOWEXP),BM_SETCHECK,BST_CHECKED,0);
            else
              SendMessage(GetDlgItem(hwndDlg,IDC_DISALLOWEXP),BM_SETCHECK,BST_UNCHECKED,0);


			SetDlgItemText(hwndDlg,IDC_VERSION,VERSION);
			return TRUE;
		
		case WM_COMMAND: 
            switch (LOWORD(wParam)) 
            { 
				case IDC_SUSER:
				  if(IsDlgButtonChecked(hwndDlg,IDC_SUSER))
				  {
					EnableWindow(GetDlgItem(hwndDlg,IDC_SUSERNAME),TRUE);
					EnableWindow(GetDlgItem(hwndDlg,IDC_SPASSWORD),TRUE);
					sUserLogin = true;
				  }
				  else
				  {
					EnableWindow(GetDlgItem(hwndDlg,IDC_SUSERNAME),FALSE);
					EnableWindow(GetDlgItem(hwndDlg,IDC_SPASSWORD),FALSE);
					sUserLogin = false;
				  }
				  break;

				// Domain management behavior
				case IDC_DOMAININTERACTION:
					// If it is now checked, enable everything
					if(IsDlgButtonChecked(hwndDlg,IDC_DOMAININTERACTION))
					{
						EnableWindow(GetDlgItem(hwndDlg,IDC_DOMAINUSER),TRUE);
						EnableWindow(GetDlgItem(hwndDlg,IDC_DOMAINPASS),TRUE);
						EnableWindow(GetDlgItem(hwndDlg,IDC_DOMAIN),TRUE);
					}
					else
					{
						// Disable the user/pass
						EnableWindow(GetDlgItem(hwndDlg,IDC_DOMAINUSER),FALSE);
						EnableWindow(GetDlgItem(hwndDlg,IDC_DOMAINPASS),FALSE);
						if(!IsDlgButtonChecked(hwndDlg,IDC_DOMAINLOGIN))
						{
							// If the domain login isnt checked either, disable the domain
							EnableWindow(GetDlgItem(hwndDlg,IDC_DOMAIN),FALSE);
						}
					}
					break;

				case IDC_DOMAINLOGIN:
					// If it is now checked, enable everything
					if(IsDlgButtonChecked(hwndDlg,IDC_DOMAINLOGIN))
						EnableWindow(GetDlgItem(hwndDlg,IDC_DOMAIN),TRUE);
					else
						if(!IsDlgButtonChecked(hwndDlg,IDC_DOMAININTERACTION))
							EnableWindow(GetDlgItem(hwndDlg,IDC_DOMAIN),FALSE);
					break;

				// User has entered something and hit OK
                case IDOK: 
					// Get the text from the edit boxes
					// store them in settingsInfo.*
					if(settingsInfo.pathPlugin)
						free(settingsInfo.pathPlugin);

					if(settingsInfo.mapPaths)
					{
					  //free(settingsInfo.mapPaths);
					  settingsInfo.mapPaths = NULL;
					}

					if(settingsInfo.pathProfile)
						free(settingsInfo.pathProfile);

					if(settingsInfo.pathMSGina)
						free(settingsInfo.pathMSGina);

					if(motdText)
						free(motdText);

					if(windowTitle)
						free(windowTitle);

					if(customLogo)
						free(customLogo);

					if(defaultDomain)
						free(defaultDomain);

					if(globalGroups)
						free(globalGroups);

					if(UserLoginScript)
					  free(UserLoginScript);

					GetDlgItemText(hwndDlg,IDC_PLUGIN,TempString,sizeof(TempString)/2-1);
                    settingsInfo.pathPlugin = _tcsdup(TempString);
					GetDlgItemText(hwndDlg,IDC_MAPS,TempString,sizeof(TempString)/2-1);
                    settingsInfo.mapPaths = _tcsdup(TempString);
					GetDlgItemText(hwndDlg,IDC_PROFILE,TempString,sizeof(TempString)/2-1); 
					settingsInfo.pathProfile = _tcsdup(TempString);                     
					GetDlgItemText(hwndDlg,IDC_MSGINA,TempString,sizeof(TempString)/2-1);
					settingsInfo.pathMSGina = _tcsdup(TempString);
					GetDlgItemText(hwndDlg,IDC_MOTD,TempString,sizeof(TempString)/2-1);
					motdText = _tcsdup(TempString);
					GetDlgItemText(hwndDlg,IDC_LOGONTITLE,TempString,sizeof(TempString)/2-1);
					windowTitle = _tcsdup(TempString);
					GetDlgItemText(hwndDlg,IDC_CUSTOMLOGO,TempString,sizeof(TempString)/2-1);
					customLogo = _tcsdup(TempString);
                    GetDlgItemText(hwndDlg,IDC_DOMAIN,TempString,sizeof(TempString)/2-1);
                    defaultDomain = _tcsdup(TempString);
                    GetDlgItemText(hwndDlg,IDC_GROUPS,TempString,sizeof(TempString)/2-1);
                    globalGroups = _tcsdup(TempString);
					GetDlgItemText(hwndDlg,IDC_SUSERNAME,TempString,sizeof(TempString)/2-1);
					sUsername = _tcsdup(TempString);
					GetDlgItemText(hwndDlg,IDC_SPASSWORD,TempString,sizeof(TempString)/2-1);
					sPassword = _tcsdup(TempString);
					GetDlgItemText(hwndDlg,IDC_LOGINSCRIPT,TempString,sizeof(TempString)/2-1);
					UserLoginScript = _tcsdup(TempString);
					GetDlgItemText(hwndDlg,IDC_DOMAINUSER,TempString,sizeof(TempString)/2-1);
					adminUsername = _tcsdup(TempString);
					GetDlgItemText(hwndDlg,IDC_DOMAINPASS,TempString,sizeof(TempString)/2-1);
					adminPassword = _tcsdup(TempString);


					if(IsDlgButtonChecked(hwndDlg,IDC_SUSER))
					  sUserLogin = true;
					else
					  sUserLogin = false;

					// Drop if sPasword is blank
                    if(sPassword[0] == '\0' && sUserLogin)
					{
					  pGinaMessageBox(IDS_PWDREQUIRED,(UINT)0,MB_OK);
					  break;
					}
					
					if(IsDlgButtonChecked(hwndDlg,IDC_debugOutput))
						debugOutput = true;
					else
						debugOutput = false;
					
					if(IsDlgButtonChecked(hwndDlg,IDC_KEEPPROFILES))
						keepProfiles = true;
					else
						keepProfiles = false;

					if(IsDlgButtonChecked(hwndDlg,IDC_PASSTHRU))
						settingsInfo.disabled = true;
					else
   						settingsInfo.disabled = false;

					if(IsDlgButtonChecked(hwndDlg,IDC_LOCKING))
   						allowLocking = true;
					else
   						allowLocking = false;

					if(IsDlgButtonChecked(hwndDlg,IDC_MAPADMIN))
   						mapAdmin = true;
					else
   						mapAdmin = false;

					if(IsDlgButtonChecked(hwndDlg,IDC_BUTTONACTION_REBOOT))
   						buttonAction = 1;
					else if(IsDlgButtonChecked(hwndDlg,IDC_BUTTONACTION_SHUTDOWN))
						buttonAction = 0;
					else
  						buttonAction = 2;

					if(IsDlgButtonChecked(hwndDlg,IDC_FORCELOCAL))
   						forceLocal = true;
					else
   						forceLocal = false;

					if(IsDlgButtonChecked(hwndDlg,IDC_HIDEPLUGIN))
						hidePlugin = true;
					else
						hidePlugin = false;

					if(IsDlgButtonChecked(hwndDlg,IDC_HIDEMOTD))
						hideMotd = true;
					else
						hideMotd = false;

                    if(IsDlgButtonChecked(hwndDlg,IDC_DISPLAY_ERRORS))
						displayErrors = true;
					else
						displayErrors = false;

					if(IsDlgButtonChecked(hwndDlg,IDC_SCRAMBLEPASS))
					  scramblePasswords = true;
					else
					  scramblePasswords = false;

					if(IsDlgButtonChecked(hwndDlg,IDC_ALLOWSPACES))
					  allowSpaces = true;
					else
					  allowSpaces = false;

                    if(IsDlgButtonChecked(hwndDlg,IDC_DISALLOWEXP))
                      disallowExpiration = true;
                    else
                      disallowExpiration = false;

					if(IsDlgButtonChecked(hwndDlg,IDC_DOMAININTERACTION))
						manageDomain = true;
					else
						manageDomain = false;

					if(IsDlgButtonChecked(hwndDlg,IDC_DOMAINLOGIN))
						includeDomain = true;
					else
						includeDomain = false;

				    if(pGinaSaveInfo(&settingsInfo))
					{
						// "Settings Saved"
					    pGinaMessageBox(IDS_SAVED,IDS_T_RESULT,MB_OK);
					} 
					break;
				case IDC_TEST:
					if(settingsInfo.pathPlugin)
						free(settingsInfo.pathPlugin);

					GetDlgItemText(hwndDlg,IDC_PLUGIN,TempString,sizeof(TempString)/2-1);                 	
					settingsInfo.pathPlugin = _tcsdup(TempString);                                     
					
					if(pGinaPluginLoad(&settingsInfo))
						pluginChangePluginSettings();

					break;
				case IDC_PLUGIN_BROWSE:
					// Browse for the plugin DLL of choice (as opposed to PGINA_LOGO_BMP)
					browseFilename = browseForFile(PGINA_PLUGIN_DLL); 
					if(browseFilename) // If it was successful, display
					{ 
					  settingsInfo.pathPlugin = browseFilename;
					  SetDlgItemText(hwndDlg,IDC_PLUGIN,settingsInfo.pathPlugin);
					}
					break;
				case IDC_LOGO_BROWSE:
					// Browse for the plugin DLL of choice (as opposed to PGINA_LOGO_BMP)
					browseFilename = browseForFile(PGINA_LOGO_BMP); 
					if(browseFilename) // If it was successful, display
					{ 
					  customLogo = browseFilename;
					  SetDlgItemText(hwndDlg,IDC_CUSTOMLOGO,customLogo);
					}
   				break;
				case IDC_LOGINSCRIPT_BROWSE:
  				    browseFilename = browseForFile(PGINA_SCRIPT_FILES);
					if(browseFilename)
					{
					  //SetDlgItemText(hwndDlg,IDC_LOGINSCRIPT,browseFilename);
					}
				break;
				case IDCANCEL:
					// Set the text blank...
					EndDialog(hwndDlg, IDOK);
					break;
            }
			return TRUE;
    } 
    return FALSE; 
}
 */

#define BUFFER_SIZE 4096

HRESULT ShowConfigDialog(void) {
    PROPSHEETHEADER propHeader;
    PROPSHEETPAGE propPages[8];

    memset(&propHeader, 0, sizeof(PROPSHEETHEADER));

    // First things first, read all our current settings
    LoadSettings();
	GetTitle();
	GetMotd();
	LoadLogo();

    // Fill out the property sheet information for the General configuration sheet
    propPages[0].dwSize         =   sizeof(PROPSHEETPAGE);
    propPages[0].dwFlags        =   PSP_DEFAULT | PSP_USETITLE;
    propPages[0].hInstance      =   pGinahInstance;
    propPages[0].pszTemplate    =   MAKEINTRESOURCE(IDD_PROP_CONFIG_GENERAL);
    propPages[0].pfnDlgProc     =   ConfigGeneralDialogProc;
    propPages[0].lParam         =   0;
    propPages[0].pfnCallback    =   NULL;
	propPages[0].pszTitle		= 	_tcsdup(TranslatedString(TEXT("IDD_PROP_CONFIG_GENERAL"),TEXT("CAPTION")));

    // Fill out the property sheet information for the Login configuration sheet
    propPages[1].dwSize         =   sizeof(PROPSHEETPAGE);
    propPages[1].dwFlags        =   PSP_USETITLE;
    propPages[1].hInstance      =   pGinahInstance;
    propPages[1].pszTemplate    =   MAKEINTRESOURCE(IDD_PROP_CONFIG_LOGIN);
    propPages[1].pfnDlgProc     =   ConfigLoginDialogProc;
    propPages[1].lParam         =   0;
    propPages[1].pfnCallback    =   NULL;
	propPages[1].pszTitle		= 	_tcsdup(TranslatedString(TEXT("IDD_PROP_CONFIG_LOGIN"),TEXT("CAPTION")));


    // Fill out the property sheet information for the Locked configuration sheet
    propPages[2].dwSize         =   sizeof(PROPSHEETPAGE);
    propPages[2].dwFlags        =   PSP_USETITLE;
    propPages[2].hInstance      =   pGinahInstance;
    propPages[2].pszTemplate    =   MAKEINTRESOURCE(IDD_PROP_CONFIG_LOCKED);
    propPages[2].pfnDlgProc     =   ConfigLockedDialogProc;
    propPages[2].lParam         =   0;
    propPages[2].pfnCallback    =   NULL;
	propPages[2].pszTitle		= 	_tcsdup(TranslatedString(TEXT("IDD_PROP_CONFIG_LOCKED"),TEXT("CAPTION")));


    // Fill out the property sheet information for the account interaction configuration sheet
    propPages[3].dwSize         =   sizeof(PROPSHEETPAGE);
    propPages[3].dwFlags        =   PSP_USETITLE;
    propPages[3].hInstance      =   pGinahInstance;
    propPages[3].pszTemplate    =   MAKEINTRESOURCE(IDD_PROP_CONFIG_ACCOUNTS);
    propPages[3].pfnDlgProc     =   ConfigAccountsDialogProc;
    propPages[3].lParam         =   0;
    propPages[3].pfnCallback    =   NULL;
	propPages[3].pszTitle		= 	_tcsdup(TranslatedString(TEXT("IDD_PROP_CONFIG_ACCOUNTS"),TEXT("CAPTION")));

    // Fill out the property sheet information for the domain interaction configuration sheet
    propPages[4].dwSize         =   sizeof(PROPSHEETPAGE);
    propPages[4].dwFlags        =   PSP_USETITLE;
    propPages[4].hInstance      =   pGinahInstance;
    propPages[4].pszTemplate    =   MAKEINTRESOURCE(IDD_PROP_CONFIG_DOMAIN);
    propPages[4].pfnDlgProc     =   ConfigDomainDialogProc;
    propPages[4].lParam         =   0;
    propPages[4].pfnCallback    =   NULL;
	propPages[4].pszTitle		= 	_tcsdup(TranslatedString(TEXT("IDD_PROP_CONFIG_DOMAIN"),TEXT("CAPTION")));

    // Fill out the property sheet information for the advanced configuration sheet
    propPages[5].dwSize         =   sizeof(PROPSHEETPAGE);
    propPages[5].dwFlags        =   PSP_USETITLE;
    propPages[5].hInstance      =   pGinahInstance;
    propPages[5].pszTemplate    =   MAKEINTRESOURCE(IDD_PROP_CONFIG_ADVANCED);
    propPages[5].pfnDlgProc     =   ConfigAdvancedDialogProc;
    propPages[5].lParam         =   0;
    propPages[5].pfnCallback    =   NULL;
	propPages[5].pszTitle		= 	_tcsdup(TranslatedString(TEXT("IDD_PROP_CONFIG_ADVANCED"),TEXT("CAPTION")));


	// Fill out the property sheet information for the profile configuration sheet
    propPages[6].dwSize         =   sizeof(PROPSHEETPAGE);
    propPages[6].dwFlags        =   PSP_USETITLE;
    propPages[6].hInstance      =   pGinahInstance;
    propPages[6].pszTemplate    =   MAKEINTRESOURCE(IDD_PROP_CONFIG_PROFILE);
    propPages[6].pfnDlgProc     =   ConfigProfileDialogProc;
    propPages[6].lParam         =   0;
    propPages[6].pfnCallback    =   NULL;
	propPages[6].pszTitle		= 	_tcsdup(TranslatedString(TEXT("IDD_PROP_CONFIG_PROFILE"),TEXT("CAPTION")));


    // Fill out the property sheet information for the about configuration sheet
    propPages[7].dwSize         =   sizeof(PROPSHEETPAGE);
    propPages[7].dwFlags        =   PSP_USETITLE;
    propPages[7].hInstance      =   pGinahInstance;
    propPages[7].pszTemplate    =   MAKEINTRESOURCE(IDD_PROP_CONFIG_ABOUT);
    propPages[7].pfnDlgProc     =   ConfigAboutDialogProc;
    propPages[7].lParam         =   0;
    propPages[7].pfnCallback    =   NULL;
	propPages[7].pszTitle		= 	_tcsdup(TranslatedString(TEXT("IDD_PROP_CONFIG_ABOUT"),TEXT("CAPTION")));

	
    // Fill out the property sheet header
    propHeader.dwSize           =   sizeof(PROPSHEETHEADER);
    propHeader.dwFlags          =   PSH_PROPSHEETPAGE | PSH_NOAPPLYNOW;
    propHeader.hwndParent       =   NULL;
    propHeader.hInstance        =   pGinahInstance;
    propHeader.hIcon            =   NULL;
    propHeader.pszIcon          =   TEXT("");
    propHeader.pszCaption       =   _tcsdup(TranslatedString(TEXT("Global"),TEXT("e19")));
    propHeader.nPages           =   sizeof(propPages) / sizeof(PROPSHEETPAGE);
    propHeader.nStartPage       =   0;
    propHeader.ppsp             =   propPages;
    propHeader.pfnCallback      =   NULL;


    if(PropertySheet(&propHeader) == -1) {
        return S_FALSE;
    }

    // Save any changes
    SaveSettings();
    return S_OK;
}

BOOL CALLBACK ConfigGeneralDialogProc(HWND hDlg, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
    wchar_t buffer[BUFFER_SIZE];
    BOOL isPluginLoaded = false;
    LPTSTR browseFilename, oldFilename;
	LPTSTR Section = TEXT("IDD_PROP_CONFIG_GENERAL");

    switch(uMessage) {
    case WM_INITDIALOG:
        // Make sure the currently selected plugin is loaded (if possible)
        if(LoadPlugin())
            isPluginLoaded = true;
		
		// SetDlgItemText(hDlg,IDC_PLUGINOPTS_BOLD,TranslatedString(Section,TEXT("1")));
		SetDlgItemText(hDlg,IDC_2,TranslatedString(Section,TEXT("2")));
		SetDlgItemText(hDlg,IDC_3,TranslatedString(Section,TEXT("3")));
		SetDlgItemText(hDlg,IDC_PLUGIN_BROWSE,TranslatedString(Section,TEXT("Browse")));
		SetDlgItemText(hDlg,IDC_PLUGIN_CONFIG,TranslatedString(Section,TEXT("Configure")));
		SetDlgItemText(hDlg,IDC_BYPASS,TranslatedString(Section,TEXT("4")));
		SetDlgItemText(hDlg,IDC_5,TranslatedString(Section,TEXT("5")));

        SendMessage(GetDlgItem(hDlg,IDC_BYPASS),BM_SETCHECK,useBypass,0);

		// Make some of our section descriptions bold
		// MakeBold(GetDlgItem(hDlg, IDC_PLUGINOPTS_BOLD), FALSE, FW_BOLD);

        // Set the text in our text boxes, and set their limits accordingly
        SetDlgItemText(hDlg,IDC_PLUGIN,settingsInfo.pathPlugin); // Plugin Path
        SendMessage(GetDlgItem(hDlg, IDC_PLUGIN), EM_SETLIMITTEXT, BUFFER_SIZE, 0);
        SetDlgItemText(hDlg,IDC_PLUGINNAME,pluginName);
        SendMessage(GetDlgItem(hDlg, IDC_ABOUTPLUGIN), EM_SETLIMITTEXT, BUFFER_SIZE, 0); // limit this one first
        if(isPluginLoaded)
            SetDlgItemText(hDlg, IDC_ABOUTPLUGIN, pluginAboutPlugin());
        else
            SetDlgItemText(hDlg, IDC_ABOUTPLUGIN, TEXT(""));


        break;
    case WM_COMMAND:
        switch(LOWORD(wParam)) {
        // Our own buttons etc here;
        case IDC_PLUGIN_BROWSE:
            // Browse for the plugin DLL of choice (as opposed to PGINA_LOGO_BMP)
			browseFilename = browseForFile(PGINA_PLUGIN_DLL); 
			if(browseFilename) // If it was successful 
			{ 
                // Save the current one
                oldFilename = settingsInfo.pathPlugin;
                settingsInfo.pathPlugin = browseFilename;
                // Try loading it
                isPluginLoaded = LoadPlugin();
                if(isPluginLoaded) {
                    SetDlgItemText(hDlg,IDC_PLUGIN,settingsInfo.pathPlugin);
                    SetDlgItemText(hDlg, IDC_ABOUTPLUGIN, pluginAboutPlugin());
                } else {
                    settingsInfo.pathPlugin = oldFilename; // return to our old one
                }
			}
            break;
        case IDC_PLUGIN_CONFIG:
            if(!isPluginLoaded)
                isPluginLoaded = LoadPlugin();
            if(isPluginLoaded)
                pluginChangePluginSettings();
            break;
        default:
            break;
        }
        break;
    case WM_NOTIFY:                        
        switch(((NMHDR FAR *)lParam)->code) {
		case PSN_SETACTIVE:
            break;
        case PSN_KILLACTIVE:
            break;
        case PSN_APPLY:
            // Free any existing data
            if(settingsInfo.pathPlugin) free(settingsInfo.pathPlugin);

            // Save data from this page
            // IDC_PLUGIN->settingsInfo.pathPlugin, IDC_MAPS->settingsInfo.mapPaths, IDC_GROUPS->globalGroups
            GetDlgItemText(hDlg,IDC_PLUGIN,buffer,BUFFER_SIZE - 1);
            settingsInfo.pathPlugin = _tcsdup(buffer);
            GetDlgItemText(hDlg,IDC_PLUGINNAME,buffer,BUFFER_SIZE - 1);
            pluginName = _tcsdup(buffer);
            useBypass = IsDlgButtonChecked(hDlg,IDC_BYPASS);
            break;
        case PSN_RESET:
            break;
        }
        break;
    default: 
        break;
    }

    return FALSE;
}

BOOL CALLBACK ConfigLoginDialogProc(HWND hDlg, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
    wchar_t buffer[BUFFER_SIZE];
    LPTSTR browseFilename;
    HBITMAP thisBMP = NULL;
	LPTSTR Section = TEXT("IDD_PROP_CONFIG_LOGIN");

    switch(uMessage) {
	case WM_INITDIALOG:
		// Translations
		SetDlgItemText(hDlg,IDC_CUSTOMLOGO_BOLD, TranslatedString(Section,TEXT("1")));
		SetDlgItemText(hDlg,IDC_2, TranslatedString(Section,TEXT("2")));
		SetDlgItemText(hDlg,IDC_3, TranslatedString(Section,TEXT("3")));
		SetDlgItemText(hDlg,IDC_MOTD_BOLD, TranslatedString(Section,TEXT("4")));
		SetDlgItemText(hDlg,IDC_HIDEMOTD, TranslatedString(Section,TEXT("5")));
		SetDlgItemText(hDlg,IDC_HIDEPLUGIN, TranslatedString(Section,TEXT("6")));
		SetDlgItemText(hDlg,IDC_BUTTON_ACTION_BOLD, TranslatedString(Section,TEXT("7")));
		SetDlgItemText(hDlg,IDC_LOGO_BROWSE,TranslatedString(Section,TEXT("Browse")));
		SetDlgItemText(hDlg,IDC_LOGO_REFRESH,TranslatedString(Section,TEXT("Refresh")));
		SetDlgItemText(hDlg,IDC_BUTTONACTION_NONE,TranslatedString(Section,TEXT("Disabled")));
		SetDlgItemText(hDlg,IDC_BUTTONACTION_REBOOT,TranslatedString(Section,TEXT("Reboot")));
		SetDlgItemText(hDlg,IDC_BUTTONACTION_SHUTDOWN,TranslatedString(Section,TEXT("Shutdown")));


		MakeBold(GetDlgItem(hDlg,IDC_CUSTOMLOGO_BOLD),FALSE,FW_BOLD);
		MakeBold(GetDlgItem(hDlg,IDC_MOTD_BOLD),FALSE,FW_BOLD);

        SetDlgItemText(hDlg,IDC_CUSTOMLOGO,customLogo); // Plugin Path
        SendMessage(GetDlgItem(hDlg, IDC_CUSTOMLOGO), EM_SETLIMITTEXT, BUFFER_SIZE, 0);

        if(LoadLogo()) {
            SendDlgItemMessage(hDlg,IDC_LOGO,STM_SETIMAGE,IMAGE_BITMAP,(LPARAM)logoBmp); 
        }

        SetDlgItemText(hDlg,IDC_LOGONTITLE,GetTitle());
        SendMessage(GetDlgItem(hDlg, IDC_LOGONTITLE), EM_SETLIMITTEXT, BUFFER_SIZE, 0);

        SetDlgItemText(hDlg,IDC_MOTD,GetMotd());
        SendMessage(GetDlgItem(hDlg, IDC_MOTD), EM_SETLIMITTEXT, BUFFER_SIZE, 0);
        SendMessage(GetDlgItem(hDlg,IDC_SAVEUSER),BM_SETCHECK,saveLastUsername,0);
        SendMessage(GetDlgItem(hDlg,IDC_HIDEMOTD),BM_SETCHECK,hideMotd,0);
        SendMessage(GetDlgItem(hDlg,IDC_HIDEPLUGIN),BM_SETCHECK,hidePlugin,0);

        // If hideMotd is true, we want to disable editingthe MOTD
        EnableWindow(GetDlgItem(hDlg,IDC_MOTD),!hideMotd);

		GetDlgItemText(hDlg,IDC_CUSTOMLOGO,buffer,BUFFER_SIZE - 1);
		if(thisBMP)
			DeleteObject(thisBMP);
		thisBMP = LoadImage((HINSTANCE)NULL,(LPCTSTR)buffer,IMAGE_BITMAP,0,0,LR_LOADFROMFILE);
		if(thisBMP) {
			SendDlgItemMessage(hDlg,IDC_LOGO,STM_SETIMAGE,IMAGE_BITMAP,(LPARAM)thisBMP); 
		} else {
			LoadLogo();
			SendDlgItemMessage(hDlg,IDC_LOGO,STM_SETIMAGE,IMAGE_BITMAP,(LPARAM)logoBmp); 
		}
                                                             
        // Setup the button action check boxes
        switch(buttonAction) {
        case 1:
            SendMessage(GetDlgItem(hDlg,IDC_BUTTONACTION_REBOOT),BM_SETCHECK,BST_CHECKED,0);
            break;
        case 0:
            SendMessage(GetDlgItem(hDlg,IDC_BUTTONACTION_SHUTDOWN),BM_SETCHECK,BST_CHECKED,0);
            break;
        default:
            SendMessage(GetDlgItem(hDlg,IDC_BUTTONACTION_NONE),BM_SETCHECK,BST_CHECKED,0);
        }
        		
        break;
    case WM_COMMAND:
        switch(LOWORD(wParam)) {
        // Our own buttons etc here;
        case IDC_BUTTONACTION_REBOOT:
            SendMessage(GetDlgItem(hDlg,IDC_BUTTONACTION_NONE),BM_SETCHECK,false,0);
            SendMessage(GetDlgItem(hDlg,IDC_BUTTONACTION_SHUTDOWN),BM_SETCHECK,false,0);
            buttonAction = 1;
            break;
        case IDC_BUTTONACTION_SHUTDOWN:
            SendMessage(GetDlgItem(hDlg,IDC_BUTTONACTION_REBOOT),BM_SETCHECK,false,0);
            SendMessage(GetDlgItem(hDlg,IDC_BUTTONACTION_NONE),BM_SETCHECK,false,0);
            buttonAction = 0;
            break;
        case IDC_BUTTONACTION_NONE:
            SendMessage(GetDlgItem(hDlg,IDC_BUTTONACTION_REBOOT),BM_SETCHECK,false,0);
            SendMessage(GetDlgItem(hDlg,IDC_BUTTONACTION_SHUTDOWN),BM_SETCHECK,false,0);
            buttonAction = 2;
            break;
        case IDC_HIDEMOTD:
            EnableWindow(GetDlgItem(hDlg,IDC_MOTD),!IsDlgButtonChecked(hDlg,IDC_HIDEMOTD));
            break;
        case IDC_LOGO_BROWSE:
            browseFilename = browseForFile(PGINA_LOGO_BMP); 
			if(browseFilename) // If it was successful, display
			{ 
                customLogo = browseFilename;
				SetDlgItemText(hDlg,IDC_CUSTOMLOGO,customLogo);
			}
        case IDC_LOGO_REFRESH:
            GetDlgItemText(hDlg,IDC_CUSTOMLOGO,buffer,BUFFER_SIZE - 1);
            if(thisBMP)
                DeleteObject(thisBMP);
            thisBMP = LoadImage((HINSTANCE)NULL,(LPCTSTR)buffer,IMAGE_BITMAP,0,0,LR_LOADFROMFILE);
            if(thisBMP) {
                SendDlgItemMessage(hDlg,IDC_LOGO,STM_SETIMAGE,IMAGE_BITMAP,(LPARAM)thisBMP); 
            } else {
                LoadLogo();
                SendDlgItemMessage(hDlg,IDC_LOGO,STM_SETIMAGE,IMAGE_BITMAP,(LPARAM)logoBmp); 
            }
            break;
        default:
            break;
        }
        break;
    case WM_NOTIFY:
        switch(((NMHDR FAR *)lParam)->code) {
        case PSN_SETACTIVE:
            break;
        case PSN_KILLACTIVE:
            break;
        case PSN_APPLY:
            if(motdText) free(motdText);
            if(windowTitle) free(windowTitle);
            if(customLogo) free(customLogo);
            // Save settings, IDC_MOTD->motdText, IDC_BUTTONACTION_????->buttonAction
            //  IDC_HIDEMOTD->hideMotd, IDC_HIDEPLUGIN->hidePlugin, IDC_LOGONTITLE->windowTitle
            //  IDC_CUSTOMLOGO->customLogo
            GetDlgItemText(hDlg,IDC_MOTD,buffer,BUFFER_SIZE - 1);
            motdText = _tcsdup(buffer);
            GetDlgItemText(hDlg,IDC_LOGONTITLE,buffer,BUFFER_SIZE - 1);
            windowTitle = _tcsdup(buffer);
            GetDlgItemText(hDlg,IDC_CUSTOMLOGO,buffer,BUFFER_SIZE - 1);
            customLogo = _tcsdup(buffer);
            hideMotd = IsDlgButtonChecked(hDlg,IDC_HIDEMOTD);
            hidePlugin = IsDlgButtonChecked(hDlg,IDC_HIDEPLUGIN);
            saveLastUsername = IsDlgButtonChecked(hDlg,IDC_SAVEUSER);
            break;
        case PSN_RESET:
            break;
        }
        break;
    default: 
        break;
    }

    return FALSE;
}

BOOL CALLBACK ConfigLockedDialogProc(HWND hDlg, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
    wchar_t buffer[BUFFER_SIZE];
	LPTSTR Section = TEXT("IDD_PROP_CONFIG_LOCKED");

    // Manipulate the following
    /**
     * allowLocking (IDC_LOCKING)
     * showUsernameAndAuth (IDC_DISPLAYUSERNAME)
     * showMachineName (IDC_DISPLAYMACHINENAME)
     * enableTask (IDC_ALLOWTASKS)
     * enableLogout (IDC_ALLOWLOGOUT)
     * enableRestart (IDC_ALLOWRESTART)
	 * trueUnlock (IDC_TRUEUNLOCK)
     */

	LPTSTR browseFilename;
    switch(uMessage) {
	case WM_INITDIALOG:
		// Translations
		SetDlgItemText(hDlg,IDC_LOCKING, TranslatedString(Section,TEXT("1")));
		SetDlgItemText(hDlg,IDC_DISPLAYUSERNAME, TranslatedString(Section,TEXT("2")));
		SetDlgItemText(hDlg,IDC_DISPLAYMACHINENAME, TranslatedString(Section,TEXT("3")));
		SetDlgItemText(hDlg,IDC_ALLOWTASKS, TranslatedString(Section,TEXT("4")));
		SetDlgItemText(hDlg,IDC_ALLOWLOGOUT, TranslatedString(Section,TEXT("5")));
		SetDlgItemText(hDlg,IDC_ALLOWRESTART, TranslatedString(Section,TEXT("6")));
		SetDlgItemText(hDlg,IDC_DISABLEPASSWORD, TranslatedString(Section,TEXT("7")));
		SetDlgItemText(hDlg,IDC_TRUEUNLOCK, TranslatedString(Section,TEXT("8")));
		SetDlgItemText(hDlg,IDC_9, TranslatedString(Section,TEXT("9")));
		SetDlgItemText(hDlg,IDC_LOGO_BROWSE_LOCKED, TranslatedString(Section,TEXT("Browse")));

        SendMessage(GetDlgItem(hDlg,IDC_LOCKING),BM_SETCHECK,allowLocking,0);
        SendMessage(GetDlgItem(hDlg,IDC_DISPLAYUSERNAME),BM_SETCHECK,showUsernameAndAuth,0);
        SendMessage(GetDlgItem(hDlg,IDC_DISPLAYMACHINENAME),BM_SETCHECK,showMachineName,0);
        SendMessage(GetDlgItem(hDlg,IDC_ALLOWTASKS),BM_SETCHECK,enableTask,0);
        SendMessage(GetDlgItem(hDlg,IDC_ALLOWLOGOUT),BM_SETCHECK,enableLogout,0);
        SendMessage(GetDlgItem(hDlg,IDC_ALLOWRESTART),BM_SETCHECK,enableRestart,0);
        SendMessage(GetDlgItem(hDlg,IDC_DISABLEPASSWORD),BM_SETCHECK,disablePassword,0);
        SendMessage(GetDlgItem(hDlg,IDC_TRUEUNLOCK),BM_SETCHECK,trueUnlock,0);
		SetDlgItemText(hDlg,IDC_CUSTOMLOGO_LOCKED,customLocked);

        SendMessage(GetDlgItem(hDlg,IDC_FLOGOUT),BM_SETCHECK,allowLockedLogout,0);
        ShowWindow(GetDlgItem(hDlg,IDC_FLOGOUT),trueUnlock ? SW_SHOW : SW_HIDE);
        break;
    case WM_COMMAND:
        switch(LOWORD(wParam)) {
		case IDC_LOGO_BROWSE_LOCKED:
            browseFilename = browseForFile(PGINA_LOGO_BMP); 
			if(browseFilename) {
                customLocked = browseFilename;
				SetDlgItemText(hDlg,IDC_CUSTOMLOGO_LOCKED,customLocked);
			}
			else
				customLocked = NULL;
			break;
        case IDC_TRUEUNLOCK:
            ShowWindow(GetDlgItem(hDlg,IDC_FLOGOUT),IsDlgButtonChecked(hDlg,IDC_TRUEUNLOCK) ? SW_SHOW : SW_HIDE);
            break;
        default:
            break;
        }
        break;
    case WM_NOTIFY:
        switch(((NMHDR FAR *)lParam)->code) {
        case PSN_SETACTIVE:
            break;
        case PSN_KILLACTIVE:
            break;
        case PSN_APPLY:
            allowLocking = IsDlgButtonChecked(hDlg,IDC_LOCKING);
            showUsernameAndAuth = IsDlgButtonChecked(hDlg,IDC_DISPLAYUSERNAME);
            showMachineName = IsDlgButtonChecked(hDlg,IDC_DISPLAYMACHINENAME);
            enableTask = IsDlgButtonChecked(hDlg,IDC_ALLOWTASKS);
            enableLogout = IsDlgButtonChecked(hDlg,IDC_ALLOWLOGOUT);
            enableRestart = IsDlgButtonChecked(hDlg,IDC_ALLOWRESTART);
			disablePassword = IsDlgButtonChecked(hDlg,IDC_DISABLEPASSWORD);
			trueUnlock = IsDlgButtonChecked(hDlg,IDC_TRUEUNLOCK);
            allowLockedLogout = IsDlgButtonChecked(hDlg,IDC_FLOGOUT);
			GetDlgItemText(hDlg,IDC_CUSTOMLOGO_LOCKED,buffer,BUFFER_SIZE - 1);
            customLocked = _tcsdup(buffer);
            break;
        case PSN_RESET:
            break;
        }
        break;
    default: 
        break;
    }

    return FALSE;
}

BOOL CALLBACK ConfigAccountsDialogProc(HWND hDlg, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
    wchar_t buffer[BUFFER_SIZE];
	LPTSTR Section = TEXT("IDD_PROP_CONFIG_ACCOUNTS");

    switch(uMessage) {
	case WM_INITDIALOG:
		// Translations
		SetDlgItemText(hDlg, IDC_LOGIN_BOLD, TranslatedString(Section,TEXT("1")));
		SetDlgItemText(hDlg, IDC_KEEPPROFILES, TranslatedString(Section,TEXT("2")));
		SetDlgItemText(hDlg, IDC_HONORLOCKED, TranslatedString(Section,TEXT("3")));
		SetDlgItemText(hDlg, IDC_FORCELOCAL, TranslatedString(Section,TEXT("4")));
		SetDlgItemText(hDlg, IDC_LOGIN_OTHER_BOLD, TranslatedString(Section,TEXT("5")));
		SetDlgItemText(hDlg, IDC_SUSER, TranslatedString(Section,TEXT("6")));
		SetDlgItemText(hDlg, IDC_1, TranslatedString(Section,TEXT("Username")));
		SetDlgItemText(hDlg, IDC_2, TranslatedString(Section,TEXT("Password")));
		SetDlgItemText(hDlg, IDC_3, TranslatedString(Section,TEXT("Domain")));
		SetDlgItemText(hDlg, IDC_CACHING, TranslatedString(Section,TEXT("IDC_CACHING")));
		SetDlgItemText(hDlg, IDC_EXPIRETIME, TranslatedString(Section,TEXT("IDC_EXPIRETIME")));
	
		wsprintf(buffer,_T("%d"),acctExpireTime);
		SetDlgItemText(hDlg, IDC_EXPIRESECONDS, buffer);

        // Make some of our section descriptions bold
        MakeBold(GetDlgItem(hDlg, IDC_LOGIN_BOLD), FALSE, FW_BOLD);
        MakeBold(GetDlgItem(hDlg, IDC_LOGIN_OTHER_BOLD), FALSE, FW_BOLD);

        SendMessage(GetDlgItem(hDlg,IDC_KEEPPROFILES),BM_SETCHECK,keepProfiles,0);
        SendMessage(GetDlgItem(hDlg,IDC_FORCELOCAL),BM_SETCHECK,forceLocal,0);
        SendMessage(GetDlgItem(hDlg,IDC_SUSER),BM_SETCHECK,sUserLogin,0);
        SendMessage(GetDlgItem(hDlg,IDC_HONORLOCKED),BM_SETCHECK,honorLockedAccounts,0);
		SendMessage(GetDlgItem(hDlg,IDC_KEEPACCOUNTS),BM_SETCHECK,keepAccount,0);
		SendMessage(GetDlgItem(hDlg,IDC_CACHING),BM_SETCHECK,acctCaching,0);

		

        // Enable or disable the single user login options accordingly
        EnableWindow(GetDlgItem(hDlg,IDC_SUSERNAME),sUserLogin && !acctCaching);
        EnableWindow(GetDlgItem(hDlg,IDC_SPASSWORD),sUserLogin && !acctCaching);
        EnableWindow(GetDlgItem(hDlg,IDC_SDOMAIN),sUserLogin && !acctCaching);

        SetDlgItemText(hDlg,IDC_SUSERNAME,sUsername);
        SendMessage(GetDlgItem(hDlg, IDC_SUSERNAME), EM_SETLIMITTEXT, BUFFER_SIZE, 0);
        SetDlgItemText(hDlg,IDC_SPASSWORD,sPassword);
        SendMessage(GetDlgItem(hDlg, IDC_SPASSWORD), EM_SETLIMITTEXT, BUFFER_SIZE, 0);
        SetDlgItemText(hDlg,IDC_SDOMAIN,sDomain);
        SendMessage(GetDlgItem(hDlg, IDC_SDOMAIN), EM_SETLIMITTEXT, BUFFER_SIZE, 0);

		if(keepProfiles) {
			SendMessage(GetDlgItem(hDlg,IDC_KEEPACCOUNTS),BM_SETCHECK,TRUE,0);
			EnableWindow(GetDlgItem(hDlg,IDC_KEEPACCOUNTS),FALSE);
		}
        
        break;
    case WM_COMMAND:
        switch(LOWORD(wParam)) {
        // Our own buttons etc here;
        case IDC_SUSER:
            // Enable or disable the single user login options accordingly
            EnableWindow(GetDlgItem(hDlg,IDC_SUSERNAME),IsDlgButtonChecked(hDlg,IDC_SUSER));
            EnableWindow(GetDlgItem(hDlg,IDC_SPASSWORD),IsDlgButtonChecked(hDlg,IDC_SUSER));
            EnableWindow(GetDlgItem(hDlg,IDC_SDOMAIN),IsDlgButtonChecked(hDlg,IDC_SUSER));

			if(IsDlgButtonChecked(hDlg,IDC_SUSER)) {
				SendMessage(GetDlgItem(hDlg,IDC_CACHING),BM_SETCHECK,FALSE,0);
				EnableWindow(GetDlgItem(hDlg,IDC_CACHING),FALSE);
				EnableWindow(GetDlgItem(hDlg,IDC_EXPIRETIME),FALSE);
				EnableWindow(GetDlgItem(hDlg,IDC_EXPIRESECONDS),FALSE);
			} else {
				EnableWindow(GetDlgItem(hDlg,IDC_CACHING),TRUE);
				EnableWindow(GetDlgItem(hDlg,IDC_EXPIRETIME),TRUE);
				EnableWindow(GetDlgItem(hDlg,IDC_EXPIRESECONDS),TRUE);
			}
            break;
		case IDC_CACHING:
			acctCaching = IsDlgButtonChecked(hDlg,IDC_CACHING);
			if(acctCaching) {
				EnableWindow(GetDlgItem(hDlg,IDC_SUSERNAME),!acctCaching);
				EnableWindow(GetDlgItem(hDlg,IDC_SPASSWORD),!acctCaching);
				EnableWindow(GetDlgItem(hDlg,IDC_SDOMAIN),!acctCaching);
				EnableWindow(GetDlgItem(hDlg,IDC_SUSER),!acctCaching);
			} else {
				EnableWindow(GetDlgItem(hDlg,IDC_SUSERNAME),IsDlgButtonChecked(hDlg,IDC_SUSER));
				EnableWindow(GetDlgItem(hDlg,IDC_SPASSWORD),IsDlgButtonChecked(hDlg,IDC_SUSER));
				EnableWindow(GetDlgItem(hDlg,IDC_SDOMAIN),IsDlgButtonChecked(hDlg,IDC_SUSER));
				EnableWindow(GetDlgItem(hDlg,IDC_SUSER),!acctCaching);
			}
			if(IsDlgButtonChecked(hDlg,IDC_SUSER) && acctCaching)
				SendMessage(GetDlgItem(hDlg,IDC_SUSER),BM_SETCHECK,!acctCaching,0);
			break;
		case IDC_KEEPPROFILES:
		case IDC_KEEPACCOUNTS:
			// If keep profiles is checked, Keep Accounts must be checked
			if(IsDlgButtonChecked(hDlg,IDC_KEEPPROFILES)) {
				SendMessage(GetDlgItem(hDlg,IDC_KEEPACCOUNTS),BM_SETCHECK,TRUE,0);
				EnableWindow(GetDlgItem(hDlg,IDC_KEEPACCOUNTS),FALSE);
			} else {
				EnableWindow(GetDlgItem(hDlg,IDC_KEEPACCOUNTS),TRUE);
			}
			break;
		
        default:
            break;
        }
        break;
    case WM_NOTIFY:
        switch(((NMHDR FAR *)lParam)->code) {
        case PSN_SETACTIVE:
            break;
        case PSN_KILLACTIVE:
            break;
        case PSN_APPLY:
            keepProfiles = IsDlgButtonChecked(hDlg,IDC_KEEPPROFILES);
            forceLocal = IsDlgButtonChecked(hDlg,IDC_FORCELOCAL);
            sUserLogin = IsDlgButtonChecked(hDlg,IDC_SUSER);
			honorLockedAccounts = IsDlgButtonChecked(hDlg,IDC_HONORLOCKED); 
			acctCaching = IsDlgButtonChecked(hDlg,IDC_CACHING);
			keepAccount = IsDlgButtonChecked(hDlg,IDC_KEEPACCOUNTS);
            GetDlgItemText(hDlg,IDC_SUSERNAME,buffer,BUFFER_SIZE - 1);
            sUsername = _tcsdup(buffer);
            GetDlgItemText(hDlg,IDC_SPASSWORD,buffer,BUFFER_SIZE - 1);
            sPassword = _tcsdup(buffer);
            GetDlgItemText(hDlg,IDC_SDOMAIN,buffer,BUFFER_SIZE - 1);
            sDomain = _tcsdup(buffer);
            GetDlgItemText(hDlg,IDC_EXPIRESECONDS,buffer,BUFFER_SIZE - 1);
			acctExpireTime = _wtoi(buffer);


			if(wcslen(sPassword) == 0 && sUserLogin) {
				//pGinaMessageBox(IDS_PWDREQUIRED,NULL,MB_OK | MB_ICONERROR);
				MessageBox(GetForegroundWindow(),TranslatedString(Section,TEXT("IDS_PWDREQUIRED")),NULL,MB_OK|MB_ICONERROR);
			}
            break;
        case PSN_RESET:
            break;
        }
        break;
    default: 
        break;
    }

    return FALSE;
}

BOOL CALLBACK ConfigDomainDialogProc(HWND hDlg, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
    wchar_t buffer[BUFFER_SIZE];
	LPTSTR Section = TEXT("IDD_PROP_CONFIG_DOMAIN");

    switch(uMessage) {
	case WM_INITDIALOG:
		// Translations
        SetDlgItemText(hDlg,IDC_DOMAININTERACTION,TranslatedString(Section,TEXT("1")));
        SetDlgItemText(hDlg,IDC_DOMAINLOGIN,TranslatedString(Section,TEXT("2")));
        SetDlgItemText(hDlg,IDC_3,TranslatedString(Section,TEXT("3")));
        SetDlgItemText(hDlg,IDC_4,TranslatedString(Section,TEXT("4")));
        SetDlgItemText(hDlg,IDC_5,TranslatedString(Section,TEXT("5")));
		SetDlgItemText(hDlg,IDC_REQDOM,TranslatedString(Section,TEXT("6")));

        // Make some of our section descriptions bold
        SendMessage(GetDlgItem(hDlg,IDC_DOMAININTERACTION),BM_SETCHECK,manageDomain,0);
        SendMessage(GetDlgItem(hDlg,IDC_DOMAINLOGIN),BM_SETCHECK,includeDomain,0);
		SendMessage(GetDlgItem(hDlg,IDC_REQDOM),BM_SETCHECK,reqDom,0);

        SetDlgItemText(hDlg,IDC_DOMAIN,defaultDomain);
        SendMessage(GetDlgItem(hDlg, IDC_DOMAIN), EM_SETLIMITTEXT, BUFFER_SIZE, 0);
        SetDlgItemText(hDlg,IDC_DOMAINUSER,adminUsername);
        SendMessage(GetDlgItem(hDlg, IDC_DOMAINUSER), EM_SETLIMITTEXT, BUFFER_SIZE, 0);
        SetDlgItemText(hDlg,IDC_DOMAINPASS,adminPassword);
        SendMessage(GetDlgItem(hDlg, IDC_DOMAINPASS), EM_SETLIMITTEXT, BUFFER_SIZE, 0);

        if(manageDomain) {
            EnableWindow(GetDlgItem(hDlg,IDC_DOMAINUSER),true);
            EnableWindow(GetDlgItem(hDlg,IDC_DOMAINPASS),true);
            EnableWindow(GetDlgItem(hDlg,IDC_DOMAIN),true);
        }
        if(includeDomain) {
            EnableWindow(GetDlgItem(hDlg,IDC_DOMAIN),true);
			EnableWindow(GetDlgItem(hDlg,IDC_REQDOM),true);
        }
        break;
    case WM_COMMAND:
        switch(LOWORD(wParam)) {
        // Our own buttons etc here;
        case IDC_DOMAININTERACTION:
            // enable the user and pass boxes
            EnableWindow(GetDlgItem(hDlg,IDC_DOMAINUSER),IsDlgButtonChecked(hDlg,IDC_DOMAININTERACTION));
            EnableWindow(GetDlgItem(hDlg,IDC_DOMAINPASS),IsDlgButtonChecked(hDlg,IDC_DOMAININTERACTION));
            if(!IsDlgButtonChecked(hDlg,IDC_DOMAINLOGIN))
                EnableWindow(GetDlgItem(hDlg,IDC_DOMAIN),IsDlgButtonChecked(hDlg,IDC_DOMAININTERACTION));
            break;
		case IDC_DOMAINLOGIN:
			EnableWindow(GetDlgItem(hDlg,IDC_REQDOM),IsDlgButtonChecked(hDlg,IDC_DOMAINLOGIN));
            if(!IsDlgButtonChecked(hDlg,IDC_DOMAININTERACTION))
                EnableWindow(GetDlgItem(hDlg,IDC_DOMAIN),IsDlgButtonChecked(hDlg,IDC_DOMAINLOGIN));
            break;
        default:
            break;
        }
        break;
    case WM_NOTIFY:
        switch(((NMHDR FAR *)lParam)->code) {
        case PSN_SETACTIVE:
            break;
        case PSN_KILLACTIVE:
            break;
        case PSN_APPLY:
            manageDomain = IsDlgButtonChecked(hDlg,IDC_DOMAININTERACTION);
            includeDomain = IsDlgButtonChecked(hDlg,IDC_DOMAINLOGIN);
			if(includeDomain) {
				reqDom = IsDlgButtonChecked(hDlg,IDC_REQDOM);
			} else {
				reqDom = false;
			}
            GetDlgItemText(hDlg,IDC_DOMAIN,buffer,BUFFER_SIZE - 1);
            defaultDomain = _tcsdup(buffer);
            GetDlgItemText(hDlg,IDC_DOMAINUSER,buffer,BUFFER_SIZE - 1);
            adminUsername = _tcsdup(buffer);
            GetDlgItemText(hDlg,IDC_DOMAINPASS,buffer,BUFFER_SIZE - 1);
            adminPassword = _tcsdup(buffer);
            break;
        case PSN_RESET:
            break;
        }
        break;
    default: 
        break;
    }

    return FALSE;
}

BOOL CALLBACK ConfigAdvancedDialogProc(HWND hDlg, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
    wchar_t buffer[BUFFER_SIZE];
	LPTSTR Section = TEXT("IDD_PROP_CONFIG_ADVANCED");

	/*
	[IDD_PROP_CONFIG_ADVANCED]
	Caption=Advanced
	1=Excessive output
	2=Disable all event logging
	3=Always display plugin errors
	4=Map admins
	5=Enable 'Passthru' mode
	6=Scramble passwords on logout
	7=Allow spaces in usernames
	8=Do not allow expiration of accounts
	9=Enable Terminal Services and Remote Desktop support
	10=Gina Chain
	11=Create and Upload a Bug Report
	12=Respect automatic login registry information
	13=Encrypt registry keys
	14=Autologin using plugin
 	15=Show autologin errors
	16=Show Quick Login Button
	*/

    // Configure
    //  debugOutput
    //  displayErrors
    //  mapAdmin
    //  disabled
    //  scramblePasswords
    //  allowSpaces
    //  disallowExpiration

    switch(uMessage) {
	case WM_INITDIALOG:
		// Translations
		SetDlgItemText(hDlg,IDC_DEBUGOUTPUT,TranslatedString(Section,TEXT("1")));
		SetDlgItemText(hDlg,IDC_DISABLELOGS,TranslatedString(Section,TEXT("2")));
		SetDlgItemText(hDlg,IDC_DISPLAY_ERRORS,TranslatedString(Section,TEXT("3")));
		SetDlgItemText(hDlg,IDC_MAPADMIN,TranslatedString(Section,TEXT("4")));
		SetDlgItemText(hDlg,IDC_PASSTHRU,TranslatedString(Section,TEXT("5")));
		SetDlgItemText(hDlg,IDC_SCRAMBLEPASS,TranslatedString(Section,TEXT("6")));
		SetDlgItemText(hDlg,IDC_ALLOWSPACES,TranslatedString(Section,TEXT("7")));
		SetDlgItemText(hDlg,IDC_DISALLOWEXP,TranslatedString(Section,TEXT("8")));
		SetDlgItemText(hDlg,IDC_HOOKWLX,TranslatedString(Section,TEXT("9")));
		SetDlgItemText(hDlg,IDC_10,TranslatedString(Section,TEXT("10")));
		SetDlgItemText(hDlg,IDC_TROUBLESHOOT,TranslatedString(Section,TEXT("11")));
		SetDlgItemText(hDlg,IDC_AUTOLOGIN,TranslatedString(Section,TEXT("12")));
		SetDlgItemText(hDlg,IDC_ENCRYPT,TranslatedString(Section,TEXT("13")));
		SetDlgItemText(hDlg,IDC_AUTOLOGIN2,TranslatedString(Section,TEXT("14")));
		SetDlgItemText(hDlg,IDC_SHOWAUTOERROR,TranslatedString(Section,TEXT("15")));
		SetDlgItemText(hDlg,IDC_QUICKOPT,TranslatedString(Section,TEXT("16")));

        // Make some of our section descriptions bold
        SendMessage(GetDlgItem(hDlg,IDC_DEBUGOUTPUT),BM_SETCHECK,debugOutput,0);
        SendMessage(GetDlgItem(hDlg,IDC_DISPLAY_ERRORS),BM_SETCHECK,displayErrors,0);
        SendMessage(GetDlgItem(hDlg,IDC_MAPADMIN),BM_SETCHECK,mapAdmin,0);
        SendMessage(GetDlgItem(hDlg,IDC_PASSTHRU),BM_SETCHECK,settingsInfo.disabled,0);
        SendMessage(GetDlgItem(hDlg,IDC_SCRAMBLEPASS),BM_SETCHECK,scramblePasswords,0);
        SendMessage(GetDlgItem(hDlg,IDC_ALLOWSPACES),BM_SETCHECK,allowSpaces,0);
        SendMessage(GetDlgItem(hDlg,IDC_DISALLOWEXP),BM_SETCHECK,disallowExpiration,0);
        SendMessage(GetDlgItem(hDlg,IDC_DISABLELOGS),BM_SETCHECK,disableAllEventLogs,0);
        SendMessage(GetDlgItem(hDlg,IDC_HOOKWLX),BM_SETCHECK,hookWlx,0);
		SendMessage(GetDlgItem(hDlg,IDC_AUTOLOGIN),BM_SETCHECK,honorAutoLogin,0);
		SendMessage(GetDlgItem(hDlg,IDC_ENCRYPT),BM_SETCHECK,encryptSettings,0);
		SendMessage(GetDlgItem(hDlg,IDC_QUICKOPT),BM_SETCHECK,quickLogin,0);

		EnableWindow(GetDlgItem(hDlg,IDC_USEREDIT),pluginAutoLogin);
		EnableWindow(GetDlgItem(hDlg,IDC_PWDEDIT),pluginAutoLogin);                   
 		EnableWindow(GetDlgItem(hDlg,IDC_SHOWAUTOERROR),pluginAutoLogin);

        EnableWindow(GetDlgItem(hDlg,IDC_QUICKUSER),quickLogin);
		EnableWindow(GetDlgItem(hDlg,IDC_QUICKPASS),quickLogin);
		EnableWindow(GetDlgItem(hDlg,IDC_QUICKCAPTION),quickLogin);
        
		SetDlgItemText(hDlg,IDC_USEREDIT,palUsername);                                
		SetDlgItemText(hDlg,IDC_PWDEDIT,palPassword);                                 
		SendMessage(GetDlgItem(hDlg, IDC_USEREDIT), EM_SETLIMITTEXT, BUFFER_SIZE, 0);
		SendMessage(GetDlgItem(hDlg, IDC_PWDEDIT), EM_SETLIMITTEXT, BUFFER_SIZE, 0);  
		SendMessage(GetDlgItem(hDlg,IDC_AUTOLOGIN2),BM_SETCHECK,pluginAutoLogin,0);   
		SendMessage(GetDlgItem(hDlg,IDC_SHOWAUTOERROR),BM_SETCHECK,palShowError,0);
        SetDlgItemText(hDlg,IDC_MSGINA,settingsInfo.pathMSGina);
        SendMessage(GetDlgItem(hDlg, IDC_MSGINA), EM_SETLIMITTEXT, BUFFER_SIZE, 0);

		SetDlgItemText(hDlg,IDC_QUICKUSER,quickUser);
		SetDlgItemText(hDlg,IDC_QUICKPASS,quickPass);
		SetDlgItemText(hDlg,IDC_QUICKCAPTION,quickCaption);
        break;
    case WM_COMMAND:
        switch(LOWORD(wParam)) {
        // Our own buttons etc here;
        case IDC_TROUBLESHOOT:
            sendTroubleshootingReport();
            break;
        case IDC_DISABLELOGS:
            if(IsDlgButtonChecked(hDlg,IDC_DISABLELOGS) && IsDlgButtonChecked(hDlg, IDC_DEBUGOUTPUT))
                SendMessage(GetDlgItem(hDlg,IDC_DEBUGOUTPUT),BM_SETCHECK,false,0);
            break;
        case IDC_DEBUGOUTPUT:
            if(IsDlgButtonChecked(hDlg,IDC_DISABLELOGS) && IsDlgButtonChecked(hDlg, IDC_DEBUGOUTPUT))
                SendMessage(GetDlgItem(hDlg,IDC_DISABLELOGS),BM_SETCHECK,false,0);
            break;
		case IDC_AUTOLOGIN2:                                                                    
			EnableWindow(GetDlgItem(hDlg,IDC_USEREDIT),IsDlgButtonChecked(hDlg,IDC_AUTOLOGIN2));
			EnableWindow(GetDlgItem(hDlg,IDC_PWDEDIT),IsDlgButtonChecked(hDlg,IDC_AUTOLOGIN2)); 
			EnableWindow(GetDlgItem(hDlg,IDC_SHOWAUTOERROR),IsDlgButtonChecked(hDlg,IDC_AUTOLOGIN2));
			break;  
		case IDC_QUICKOPT:
			EnableWindow(GetDlgItem(hDlg,IDC_QUICKUSER),IsDlgButtonChecked(hDlg,IDC_QUICKOPT));
            EnableWindow(GetDlgItem(hDlg,IDC_QUICKPASS),IsDlgButtonChecked(hDlg,IDC_QUICKOPT));
			EnableWindow(GetDlgItem(hDlg,IDC_QUICKCAPTION),IsDlgButtonChecked(hDlg,IDC_QUICKOPT));
			break;
        default:
            break;
        }
        break;
    case WM_NOTIFY:
        switch(((NMHDR FAR *)lParam)->code) {
        case PSN_SETACTIVE:
            break;
        case PSN_KILLACTIVE:
            break;
        case PSN_APPLY:
            debugOutput = IsDlgButtonChecked(hDlg,IDC_DEBUGOUTPUT);
            displayErrors = IsDlgButtonChecked(hDlg,IDC_DISPLAY_ERRORS);
            mapAdmin = IsDlgButtonChecked(hDlg,IDC_MAPADMIN);
            settingsInfo.disabled = IsDlgButtonChecked(hDlg,IDC_PASSTHRU);
            scramblePasswords = IsDlgButtonChecked(hDlg,IDC_SCRAMBLEPASS);
            allowSpaces = IsDlgButtonChecked(hDlg,IDC_ALLOWSPACES);
            disallowExpiration = IsDlgButtonChecked(hDlg,IDC_DISALLOWEXP);
            disableAllEventLogs = IsDlgButtonChecked(hDlg,IDC_DISABLELOGS);
            hookWlx = IsDlgButtonChecked(hDlg,IDC_HOOKWLX);
			honorAutoLogin = IsDlgButtonChecked(hDlg,IDC_AUTOLOGIN);
			encryptSettings = IsDlgButtonChecked(hDlg,IDC_ENCRYPT);
            GetDlgItemText(hDlg,IDC_MSGINA,buffer,BUFFER_SIZE - 1);
            settingsInfo.pathMSGina = _tcsdup(buffer);
			pluginAutoLogin = IsDlgButtonChecked(hDlg,IDC_AUTOLOGIN2); 
			GetDlgItemText(hDlg,IDC_USEREDIT,buffer,BUFFER_SIZE-1);    
			palUsername = _tcsdup(buffer);                            
			GetDlgItemText(hDlg,IDC_PWDEDIT,buffer,BUFFER_SIZE-1);
			palPassword = _tcsdup(buffer);
			palShowError = IsDlgButtonChecked(hDlg,IDC_SHOWAUTOERROR);
			quickLogin = IsDlgButtonChecked(hDlg,IDC_QUICKOPT);
			GetDlgItemText(hDlg,IDC_QUICKUSER,buffer,BUFFER_SIZE-1);
			quickUser = _tcsdup(buffer);
			GetDlgItemText(hDlg,IDC_QUICKPASS,buffer,BUFFER_SIZE-1);
			quickPass = _tcsdup(buffer);
			GetDlgItemText(hDlg,IDC_QUICKCAPTION,buffer,BUFFER_SIZE-1);
			quickCaption = _tcsdup(buffer);

            break;
        case PSN_RESET:
            break;
        }
        break;
    default: 
        break;
    }

    return FALSE;
}

BOOL CALLBACK ConfigAboutDialogProc(HWND hDlg, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
//    wchar_t buffer[BUFFER_SIZE];
	LPTSTR Section = TEXT("IDD_PROP_CONFIG_ABOUT");

    switch(uMessage) {
	case WM_INITDIALOG:
		//Translations
		SetDlgItemText(hDlg, IDC_WEB_BOLD, TranslatedString(Section,TEXT("1")));
		SetDlgItemText(hDlg, IDC_WEB_BOLD2, TranslatedString(Section,TEXT("2")));
		SetDlgItemText(hDlg, IDC_WEB_BOLD4, TranslatedString(Section,TEXT("3")));
		SetDlgItemText(hDlg, IDC_WEB_BOLD5, TranslatedString(Section,TEXT("4")));

        // Make some of our section descriptions bold
        MakeBold(GetDlgItem(hDlg, IDC_WEB_BOLD), FALSE, FW_BOLD);
        MakeBold(GetDlgItem(hDlg, IDC_WEB_BOLD2), FALSE, FW_BOLD);
        MakeBold(GetDlgItem(hDlg, IDC_WEB_BOLD3), FALSE, FW_BOLD);
        MakeBold(GetDlgItem(hDlg, IDC_WEB_BOLD4), FALSE, FW_BOLD);
        MakeBold(GetDlgItem(hDlg, IDC_WEB_BOLD5), FALSE, FW_BOLD);
        SetDlgItemText(hDlg,IDC_VERSION,VERSION);

        break;
    case WM_COMMAND:
        switch(LOWORD(wParam)) {
        // Our own buttons etc here;
        default:
            break;
        }
        break;
    case WM_NOTIFY:
        switch(((NMHDR FAR *)lParam)->code) {
        case PSN_SETACTIVE:
            break;
        case PSN_KILLACTIVE:
            break;
        case PSN_APPLY:
            break;
        case PSN_RESET:
            break;
        }
        break;
    default: 
        break;
    }

    return FALSE;
}

BOOL CALLBACK ConfigProfileDialogProc(HWND hDlg, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
	wchar_t buffer[BUFFER_SIZE];
	LPTSTR Section = TEXT("IDD_PROP_CONFIG_PROFILE");

	switch(uMessage) {
	case WM_INITDIALOG:
		// Translations
		SetDlgItemText(hDlg,IDC_1,TranslatedString(Section,TEXT("1")));
		SetDlgItemText(hDlg,IDC_2,TranslatedString(Section,TEXT("2")));
		SetDlgItemText(hDlg,IDC_3,TranslatedString(Section,TEXT("3")));
		SetDlgItemText(hDlg,IDC_4,TranslatedString(Section,TEXT("4")));
		SetDlgItemText(hDlg,IDC_5,TranslatedString(Section,TEXT("5")));
		SetDlgItemText(hDlg,IDC_6,TranslatedString(Section,TEXT("6")));
        SetDlgItemText(hDlg,IDC_7,TranslatedString(Section,TEXT("7")));
        SetDlgItemText(hDlg,IDC_IDLE,TranslatedString(Section,TEXT("idle")));
        SetDlgItemText(hDlg,IDC_ALLOWRETURN,TranslatedString(Section,TEXT("return")));
        SetDlgItemText(hDlg,IDC_IDLELOCKED,TranslatedString(Section,TEXT("locked")));


		SetDlgItemText(hDlg,IDC_MAPS,settingsInfo.mapPaths);
        SendMessage(GetDlgItem(hDlg, IDC_MAPS), EM_SETLIMITTEXT, BUFFER_SIZE, 0);
        SetDlgItemText(hDlg,IDC_GROUPS,globalGroups);
        SendMessage(GetDlgItem(hDlg, IDC_GROUPS), EM_SETLIMITTEXT, BUFFER_SIZE, 0);
		SetDlgItemText(hDlg,IDC_PATH_PROFILE, settingsInfo.pathProfile);
		SendMessage(GetDlgItem(hDlg, IDC_PATH_PROFILE), EM_SETLIMITTEXT, BUFFER_SIZE, 0);
		SetDlgItemText(hDlg,IDC_PATH_PROFILE_DEFAULT, settingsInfo.pathProfileDefault);
		SendMessage(GetDlgItem(hDlg, IDC_PATH_PROFILE_DEFAULT), EM_SETLIMITTEXT, BUFFER_SIZE, 0);
		SetDlgItemText(hDlg,IDC_LOGIN_SCRIPT, UserLoginScript);
		SendMessage(GetDlgItem(hDlg, IDC_LOGIN_SCRIPT), EM_SETLIMITTEXT, BUFFER_SIZE, 0);
        SendMessage(GetDlgItem(hDlg,IDC_IDLE),BM_SETCHECK,idleLogoff,0);
        EnableWindow(GetDlgItem(hDlg,IDC_GRACE),idleLogoff);
		wsprintf(buffer,_T("%d"),defaultSessionTimeout);
		SetDlgItemText(hDlg,IDC_DEFAULTTIMEOUT, buffer);
		SendMessage(GetDlgItem(hDlg, IDC_DEFAULTTIMEOUT), EM_SETLIMITTEXT, BUFFER_SIZE, 0);
        SendMessage(GetDlgItem(hDlg,IDC_ALLOWRETURN),BM_SETCHECK,idleAllowReturn,0);
        SendMessage(GetDlgItem(hDlg,IDC_IDLELOCKED),BM_SETCHECK,idleFromLocked,0);
		SendMessage(GetDlgItem(hDlg,IDC_ROAM_PROFILE),BM_SETCHECK,roamingProfile,0);


        wsprintf(buffer,_T("%d"),idleTime);
        SetDlgItemText(hDlg,IDC_GRACE,buffer);
        SendMessage(GetDlgItem(hDlg, IDC_GRACE), EM_SETLIMITTEXT, BUFFER_SIZE, 0);
        EnableWindow(GetDlgItem(hDlg,IDC_GRACE),IsDlgButtonChecked(hDlg,IDC_IDLE));
        EnableWindow(GetDlgItem(hDlg,IDC_ALLOWRETURN),IsDlgButtonChecked(hDlg,IDC_IDLE));
        EnableWindow(GetDlgItem(hDlg,IDC_IDLELOCKED),IsDlgButtonChecked(hDlg,IDC_IDLE));

        break;
    case WM_COMMAND:
        switch(LOWORD(wParam)) {
        case IDC_IDLE:
            EnableWindow(GetDlgItem(hDlg,IDC_GRACE),IsDlgButtonChecked(hDlg,IDC_IDLE));
            EnableWindow(GetDlgItem(hDlg,IDC_ALLOWRETURN),IsDlgButtonChecked(hDlg,IDC_IDLE));
            EnableWindow(GetDlgItem(hDlg,IDC_IDLELOCKED),IsDlgButtonChecked(hDlg,IDC_IDLE));
            break;
        // Our own buttons etc here;
        default:
            break;
        }
        break;
    case WM_NOTIFY:
        switch(((NMHDR FAR *)lParam)->code) {
        case PSN_SETACTIVE:
            break;
        case PSN_KILLACTIVE:
            break;
        case PSN_APPLY:
			if(settingsInfo.mapPaths) free(settingsInfo.mapPaths);
            if(globalGroups) free(globalGroups);
			if(settingsInfo.pathProfile) free(settingsInfo.pathProfile);
			if(settingsInfo.pathProfileDefault) free(settingsInfo.pathProfileDefault);
			if(UserLoginScript) free(UserLoginScript);

			GetDlgItemText(hDlg,IDC_MAPS,buffer,BUFFER_SIZE - 1);
            settingsInfo.mapPaths = _tcsdup(buffer);
            GetDlgItemText(hDlg,IDC_GROUPS,buffer,BUFFER_SIZE - 1);
            globalGroups = _tcsdup(buffer);
			GetDlgItemText(hDlg,IDC_PATH_PROFILE,buffer,BUFFER_SIZE - 1);
            settingsInfo.pathProfile = _tcsdup(buffer);
			GetDlgItemText(hDlg,IDC_PATH_PROFILE_DEFAULT,buffer,BUFFER_SIZE - 1);
            settingsInfo.pathProfileDefault = _tcsdup(buffer);
			GetDlgItemText(hDlg,IDC_LOGIN_SCRIPT,buffer,BUFFER_SIZE - 1);
            UserLoginScript = _tcsdup(buffer);
			GetDlgItemText(hDlg,IDC_DEFAULTTIMEOUT,buffer,BUFFER_SIZE - 1);
			defaultSessionTimeout = _wtoi(buffer);
            GetDlgItemText(hDlg,IDC_GRACE,buffer,BUFFER_SIZE - 1);
            idleTime = _wtoi(buffer);
            idleLogoff = IsDlgButtonChecked(hDlg,IDC_IDLE);
            idleAllowReturn = IsDlgButtonChecked(hDlg,IDC_ALLOWRETURN);
            idleFromLocked = IsDlgButtonChecked(hDlg,IDC_IDLELOCKED);
			roamingProfile = IsDlgButtonChecked(hDlg,IDC_ROAM_PROFILE);
            break;
        case PSN_RESET:
            break;
        }
        break;
    default: 
        break;
    }

    return FALSE;
}

/*
	@author Deakin University
	
	Taken from NIS Gina project from deakin university, centers a given window on the 
	screen.
*/
VOID CenterWindow(HWND hwnd)
{
  RECT    rect;
  LONG    dx, dy;
  LONG    dxParent, dyParent;
  LONG    Style;

  GetWindowRect(hwnd, &rect);

  dx = rect.right - rect.left;
  dy = rect.bottom - rect.top;

  Style = GetWindowLong(hwnd, GWL_STYLE);
  if ((Style & WS_CHILD) == 0) 
    {
      dxParent = GetSystemMetrics(SM_CXSCREEN);
      dyParent = GetSystemMetrics(SM_CYSCREEN);
    } 
  else 
    {
      HWND    hwndParent;
      RECT    rectParent;

      hwndParent = GetParent(hwnd);
      if (hwndParent == NULL) 
	{
	  hwndParent = GetDesktopWindow();
	}

      GetWindowRect(hwndParent, &rectParent);

      dxParent = rectParent.right - rectParent.left;
      dyParent = rectParent.bottom - rectParent.top;
    }

  rect.left = (dxParent - dx) / 2;
  rect.top  = (dyParent - dy) / 3;

  SetWindowPos(hwnd, HWND_TOPMOST, rect.left, rect.top, 0, 0, SWP_NOSIZE);

  SetForegroundWindow(hwnd);
}

HRESULT MakeBold(HWND hWnd, BOOL fSize, LONG lfWeight)
{
	LOGFONT *plogfont;
	HRESULT hr;
	HFONT hfont;
	HFONT hnewfont;

	hr = ERROR_SUCCESS;
	hfont = NULL;
	hnewfont = NULL;
	plogfont = NULL;
	
	hfont = (HFONT) SendMessage(hWnd, WM_GETFONT, 0, 0);
	
	if (!hfont) {
		hr = ERROR_GEN_FAILURE;
		goto Cleanup;
	}
	
	plogfont = (LOGFONT*) malloc(sizeof(LOGFONT));
	
	if (plogfont == NULL) {
		hr = ERROR_NOT_ENOUGH_MEMORY;
		goto Cleanup;
	}
	
	if (!GetObject(hfont, sizeof(LOGFONT), (LPVOID)plogfont)) {
		hr = ERROR_GEN_FAILURE;
		goto Cleanup;
	}
	
	if (abs(plogfont->lfHeight) < 24 && fSize) {
		plogfont->lfHeight = plogfont->lfHeight + (plogfont->lfHeight / 4);
	}
	
	plogfont->lfWeight = (int) lfWeight;
	
	if (!(hnewfont = CreateFontIndirect(plogfont))) {
		hr = ERROR_GEN_FAILURE;
		goto Cleanup;
	}
	
	SendMessage(hWnd, WM_SETFONT, (WPARAM) hnewfont, MAKELPARAM(TRUE, 0));
	
Cleanup:
	
	if (plogfont) {
		free(plogfont);
	}

	return HRESULT_FROM_WIN32(hr);
}

/** Callback for the troubleshooting dialog
 * @todo Add upload of information.. perhaps through php GET ?
 */
BOOL CALLBACK TSBoxProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{
    wchar_t *stateBuffer = NULL;
    wchar_t name[BUFFER_SIZE];
    wchar_t email[BUFFER_SIZE];
    wchar_t org[BUFFER_SIZE];
    wchar_t desc[BUFFER_SIZE];

	LPTSTR Section = TEXT("IDD_TROUBLESHOOTING");
    
	switch(message)
	{
	case WM_INITDIALOG:
		CenterWindow(hDlg);

		// Translation
		SetWindowText(hDlg,TranslatedString(Section,TEXT("Caption")));
		SetDlgItemText(hDlg, IDC_DESC, TranslatedString(Section,TEXT("6")));
		SetDlgItemText(hDlg, IDC_1, TranslatedString(Section,TEXT("1")));
		SetDlgItemText(hDlg, IDC_2, TranslatedString(Section,TEXT("2")));
		SetDlgItemText(hDlg, IDC_3, TranslatedString(Section,TEXT("3")));
		SetDlgItemText(hDlg, IDC_4, TranslatedString(Section,TEXT("4")));
		SetDlgItemText(hDlg, IDC_5, TranslatedString(Section,TEXT("5")));
		SetDlgItemText(hDlg, ID_SUBMIT, TranslatedString(Section,TEXT("Submit")));
		SetDlgItemText(hDlg, IDCANCEL, TranslatedString(Section,TEXT("Cancel")));

        stateBuffer = malloc(sizeof(wchar_t) * 64000);
        dumpState(stateBuffer,sizeof(wchar_t) * 64000);
        SetDlgItemText(hDlg,IDC_REPORT,stateBuffer);
        SendMessage(GetDlgItem(hDlg, IDC_DESC), EM_SETLIMITTEXT, BUFFER_SIZE, 0);
        SendMessage(GetDlgItem(hDlg, IDC_DESCRIPTION), EM_SETLIMITTEXT, BUFFER_SIZE, 0);
        //SetDlgItemText(hDlg,IDC_DESC,TSTEXT);
        free(stateBuffer);
        SetFocus(GetDlgItem(hDlg, ID_SUBMIT));
        return TRUE;

	case WM_COMMAND: 
		switch(LOWORD(wParam))
		{
        case ID_SUBMIT:
            GetDlgItemText(hDlg,IDC_FULLNAME,name,BUFFER_SIZE - 1);
            GetDlgItemText(hDlg,IDC_EMAIL,email,BUFFER_SIZE - 1);
            GetDlgItemText(hDlg,IDC_ORG,org,BUFFER_SIZE - 1);
            GetDlgItemText(hDlg,IDC_DESCRIPTION,desc,BUFFER_SIZE - 1);
            stateBuffer = malloc(sizeof(wchar_t) * 64000);
            dumpState(stateBuffer,sizeof(wchar_t) * 64000);
            SubmitReport(name,email,org,stateBuffer,desc);
            free(stateBuffer);
        case IDCANCEL:
            EndDialog(hDlg, 0);
            break;
        default:
            break;
		}
		return TRUE;
	} 
	return FALSE; 
}


// Redirection for dialog box stuff
void HookWlxDialogBoxParam (PVOID pWinlogonFunctions)
{
	// Save the existing callback
	pfWlxDialogBoxParam = ((PWLX_DISPATCH_VERSION_1_0) pWinlogonFunctions)->WlxDialogBoxParam;
	// And add our own
    ((PWLX_DISPATCH_VERSION_1_0) pWinlogonFunctions)->WlxDialogBoxParam = MyWlxDialogBoxParam;
}

// Redirected WlxDialogBoxParam() function.
int WINAPI MyWlxDialogBoxParam (HANDLE hWlx, HANDLE hInst, LPWSTR lpszTemplate, HWND hwndOwner, DLGPROC dlgprc, LPARAM  dwInitParam)
{
   if (!HIWORD(lpszTemplate))
   {
	  // Make sure we only hook the dialog we want
      switch ((DWORD) lpszTemplate)
      {
         case IDD_WLXLOGGEDOUTSAS_DIALOG:
         {
			// Save out 
            pfWlxLoggedOutSASDlgProc = dlgprc;
			// And redirect through our own callback
            return pfWlxDialogBoxParam(hWlx, hInst, lpszTemplate, hwndOwner, MyWlxLoggedOutSASDlgProc, dwInitParam);
         }
	  }
   }

   // Just funnel the rest through
   return pfWlxDialogBoxParam(hWlx, hInst, lpszTemplate, hwndOwner, dlgprc, dwInitParam);
}

//
// Redirected WlxLoggedOutSASDlgProc().
//

BOOL CALLBACK MyWlxLoggedOutSASDlgProc (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
   BOOL bResult;
   
   // Pass on to MSGINA first.
   bResult = pfWlxLoggedOutSASDlgProc(hwndDlg, uMsg, wParam, lParam);

   // Only do this if the user turned on the option
   if(hookWlx && !usingAutoLogin && !settingsInfo.disabled) {
	   if (uMsg == WM_INITDIALOG)
	   {
		   // Set the username/password
		   if(!sUserLogin) { 	// If single user is turned off, OR its turned on but we should use settingsInfo
			   SetDlgItemText(hwndDlg,IDC_MSGINA_USERNAME,settingsInfo.Username);
			   SetDlgItemText(hwndDlg,IDC_MSGINA_PASSWORD,settingsInfo.Password);
		   } else {
			   if(useSLoginCredentials) {
				   SetDlgItemText(hwndDlg,IDC_MSGINA_USERNAME,sUsername);
				   SetDlgItemText(hwndDlg,IDC_MSGINA_PASSWORD,sPassword);
			   } else {
				   SetDlgItemText(hwndDlg,IDC_MSGINA_USERNAME,settingsInfo.Username);
				   SetDlgItemText(hwndDlg,IDC_MSGINA_PASSWORD,settingsInfo.Password);
			   }
		   }
		   // And "hit" OK ;)
		   SendMessage(hwndDlg,WM_COMMAND,1,1);
	
	   }
   }

   return bResult;
}
