/**
	@file 	wlxFuncs.c
	@brief 	GINA Interface functionality
*/
/*
pGina - A modular GINA Interface for Windows
Copyright (C) 2002 Nathan Yocom, Michael Wright

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Email: nate.yocom@xpasystems.com
Web: http://pgina.xpasystems.com
Snail Mail:
  Nathan Yocom 
  9 Evergreen Farms Rd.
  Scarborough, ME 04074
  Phone: 207-450-4948
*/

#include <windows.h>
#include <winwlx.h>
#include <stdio.h>
#include <Winreg.h>
#include <lm.h>
#include "PGina.h"
#include "resource.h"
//
// Hook into the real MSGINA.
//

BOOL MyInitialize (HINSTANCE hDll, DWORD dwWlxVersion)
{
	//
	// Get pointers to all of the WLX functions in the real MSGINA.
	//
	pfWlxInitialize = 
	(PFWLXINITIALIZE) GetProcAddress(hDll, "WlxInitialize");
	if(!pfWlxInitialize)
	{
		return FALSE;
	}

	pfWlxDisplaySASNotice =
	(PFWLXDISPLAYSASNOTICE) GetProcAddress(hDll, "WlxDisplaySASNotice");
	if(!pfWlxDisplaySASNotice)
	{
		return FALSE;
	}

	pfWlxLoggedOutSAS = 
	(PFWLXLOGGEDOUTSAS) GetProcAddress(hDll, "WlxLoggedOutSAS");
	if(!pfWlxLoggedOutSAS)
	{
		return FALSE;
	}

	pfWlxActivateUserShell =
	(PFWLXACTIVATEUSERSHELL) GetProcAddress(hDll, "WlxActivateUserShell");
	if(!pfWlxActivateUserShell)
	{
		return FALSE;
	}

	pfWlxLoggedOnSAS =
	(PFWLXLOGGEDONSAS) GetProcAddress(hDll, "WlxLoggedOnSAS");
	if(!pfWlxLoggedOnSAS)
	{
		return FALSE;
	}

	pfWlxDisplayLockedNotice =
	(PFWLXDISPLAYLOCKEDNOTICE) GetProcAddress(hDll, "WlxDisplayLockedNotice");
	if(!pfWlxDisplayLockedNotice)
	{
		return FALSE;
	}

	pfWlxIsLockOk = 
	(PFWLXISLOCKOK) GetProcAddress(hDll, "WlxIsLockOk");
	if(!pfWlxIsLockOk)
	{
		return FALSE;
	}

	pfWlxWkstaLockedSAS =
	(PFWLXWKSTALOCKEDSAS) GetProcAddress(hDll, "WlxWkstaLockedSAS");
	if(!pfWlxWkstaLockedSAS)
	{
		return FALSE;
	}

	pfWlxIsLogoffOk = 
	(PFWLXISLOGOFFOK) GetProcAddress(hDll, "WlxIsLogoffOk");
	if(!pfWlxIsLogoffOk)
	{
		return FALSE;
	}

	pfWlxLogoff = 
	(PFWLXLOGOFF) GetProcAddress(hDll, "WlxLogoff");
	if(!pfWlxLogoff)
	{
		return FALSE;
	}

	pfWlxShutdown = 
	(PFWLXSHUTDOWN) GetProcAddress(hDll, "WlxShutdown");
	if(!pfWlxShutdown)
	{
		return FALSE;
	}

	//
	// Load functions for version 1.1 as necessary.
	//
	if(dwWlxVersion > WLX_VERSION_1_0)
	{
		pfWlxStartApplication = 
		(PFWLXSTARTAPPLICATION) GetProcAddress(hDll, "WlxStartApplication");
		if(!pfWlxStartApplication)
		{
			return FALSE;
		}

		pfWlxScreenSaverNotify = 
		(PFWLXSCREENSAVERNOTIFY) GetProcAddress(hDll, "WlxScreenSaverNotify");
		if(!pfWlxScreenSaverNotify)
		{
			return FALSE;
		}
	}

	//
	// Load functions for version 1.3 as necessary.
	//
	if(dwWlxVersion > WLX_VERSION_1_2)
	{
		pfWlxNetworkProviderLoad = 
		(PFWLXNETWORKPROVIDERLOAD) 
		GetProcAddress(hDll, "WlxNetworkProviderLoad");
		if(!pfWlxNetworkProviderLoad)
		{
			return FALSE;
		}

		pfWlxDisplayStatusMessage =
		(PFWLXDISPLAYSTATUSMESSAGE) 
		GetProcAddress(hDll, "WlxDisplayStatusMessage");
		if(!pfWlxDisplayStatusMessage)
		{
			return FALSE;
		}

		pfWlxGetStatusMessage =
		(PFWLXGETSTATUSMESSAGE)
		GetProcAddress(hDll, "WlxGetStatusMessage");
		if(!pfWlxGetStatusMessage)
		{
			return FALSE;
		}

		pfWlxRemoveStatusMessage =
		(PFWLXREMOVESTATUSMESSAGE)
		GetProcAddress(hDll, "WlxRemoveStatusMessage");
		if(!pfWlxRemoveStatusMessage)
		{
			return FALSE;
		}


		// Terminal Services Additions
	}

	// 1.4 (XP)
	if(dwWlxVersion > WLX_VERSION_1_3)
	{
		pfWlxGetConsoleSwitchCredentials = 
		(PFWLXGETCONSOLESWITCHCREDENTIALS)
		GetProcAddress(hDll, "WlxGetConsoleSwitchCredentials");
		if(!pfWlxGetConsoleSwitchCredentials)
		{
			return FALSE;
		}

		pfWlxReconnectNotify = (PFWLXRECONNECTNOTIFY) GetProcAddress(hDll,"WlxReconnectNotify");
		if(!pfWlxReconnectNotify)
			return FALSE;

		pfWlxDisconnectNotify = (PFWLXDISCONNECTNOTIFY) GetProcAddress(hDll,"WlxDisconnectNotify");
		if(!pfWlxDisconnectNotify)
			return FALSE;
	}
	//
	// Load functions for newer version here...
	//

	// Get windows revision
	windowsVersion = getWindowsVersion();
	EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("Windows Version: %d"),windowsVersion);

    isLoggedIn = FALSE;
	logInTime = 0;
	sessionTimeLeft = 0;
	TimerID = 0;

	//
	// Everything loaded OK.
	//
	return TRUE;
}


BOOL 
WINAPI 
WlxNegotiate (DWORD   dwWinlogonVersion,
			  DWORD * pdwDllVersion)
{
	HINSTANCE hDll;
	DWORD dwWlxVersion = GINASTUB_VERSION;

	//
	// Load MSGINA.DLL.
	//
	if(!(hDll = LoadLibrary(settingsInfo.pathMSGina)))
		if(!(hDll = LoadLibrary(REALGINA_PATH)))
		{
			return FALSE;
		}

		//
		// Get pointers to WlxNegotiate function in the real MSGINA.
		//
	pfWlxNegotiate = (PFWLXNEGOTIATE) GetProcAddress(hDll, "WlxNegotiate");
	if(!pfWlxNegotiate)
	{
		return FALSE;
	}

	//
	// Handle older version of Winlogon.
	//
	if(dwWinlogonVersion < dwWlxVersion)
	{
		dwWlxVersion = dwWinlogonVersion;
	}

	//
	// Negotiate with MSGINA for version that we can support.
	//
	if(!pfWlxNegotiate(dwWlxVersion, &dwWlxVersion))
	{
		return FALSE;
	}

	//
	// Load the rest of the WLX functions from the real MSGINA.
	//
	if(!MyInitialize(hDll, dwWlxVersion))
	{
		return FALSE;
	}

	//
	// Inform Winlogon which version to use.
	//
	*pdwDllVersion = g_dwVersion = dwWlxVersion;

	return TRUE;
}


BOOL
WINAPI
WlxInitialize (LPWSTR  lpWinsta,
			   HANDLE  hWlx,
			   PVOID   pvReserved,
			   PVOID   pWinlogonFunctions,
			   PVOID * pWlxContext)
{

	//
	// Save pointer to dispatch table.
	// 
	// Note that g_pWinlogon will need to be properly casted to the 
	// appropriate version when used to call function in the dispatch 
	// table.
	//
	// For example, assuming we are at WLX_VERSION_1_3, we would call
	// WlxSasNotify() as follows:
	//
	// ((PWLX_DISPATCH_VERSION_1_3) g_pWinlogon)->WlxSasNotify(hWlx, MY_SAS);
	//
	HINSTANCE hDll;
	g_pWinlogon = pWinlogonFunctions;
	hGlobalWlx = hWlx;

	// Used to return here...
	pfWlxInitialize(lpWinsta,
					hWlx,
					pvReserved,
					pWinlogonFunctions,
					pWlxContext);


	windowsVersion = getWindowsVersion();
	if(windowsVersion == 3)	// If this is Windows XP, turn on the theme stuff
	{
		if(!(hDll = LoadLibrary(_T("shsvcs.dll"))))
		{
			fThemeWait = NULL;
			fThemeWatch = NULL;
		}


		//
		// Get pointers to WlxNegotiate function in the real MSGINA.
		//
		fThemeWait = (pThemeWait) GetProcAddress(hDll, "ThemeWaitForServiceReady");
		fThemeWatch = (pThemeWatch) GetProcAddress(hDll,"ThemeWatchForStart");

		if(fThemeWait && fThemeWatch)
		{
			EventLog(true,EVENTLOG_INFORMATION_TYPE,_T("Loaded Theme() functions and calling"));
			fThemeWait(2000);
			fThemeWatch();
			fThemeWait = NULL;
			fThemeWatch = NULL;
		}
		CloseHandle(hDll);
	}

	HookWlxDialogBoxParam(g_pWinlogon);

	return TRUE;
}


VOID
WINAPI
WlxDisplaySASNotice (PVOID pWlxContext)
{
	pfWlxDisplaySASNotice(pWlxContext);
}

BOOL blockSASmessages = FALSE;

VOID CALLBACK onTimerProc(HWND hwnd,
    UINT uMsg,
    UINT idEvent,
    DWORD dwTime
)
{
	static int last_show_point;
	int show_point;
	
	time_t cur_time;
	cur_time = time( &cur_time );
	
	sessionTimeLeft = settingsInfo.sessionTimeout + logInTime - cur_time;
	
	if ( sessionTimeLeft < 0 ) {
		show_point = -1;
		KillTimer(NULL,TimerID);
		TimerID = 0;
	}else if ( sessionTimeLeft <= 60 )
		show_point = 2;
	else if ( sessionTimeLeft <= 5*60 )
		show_point = 3;
	else 
		show_point = 0;
	
	if ( last_show_point != show_point && show_point != 0 ){
		last_show_point = show_point;
		if (!blockSASmessages){
			if(windowsVersion == WINDOWS_2000){
				((PWLX_DISPATCH_VERSION_1_3)g_pWinlogon)->WlxSasNotify(hGlobalWlx,WLX_SAS_TYPE_CTRL_ALT_DEL);
			}else{
				((PWLX_DISPATCH_VERSION_1_4)g_pWinlogon)->WlxSasNotify(hGlobalWlx,WLX_SAS_TYPE_CTRL_ALT_DEL);
			}
		}
	}
}


/**
	@brief Called when user sends SAS while logged out
	@author	Nathan Yocom
	
	This is the function that is called by winlogon upon initial boot, 
	and anytime the SAS (CTRL+ALT+DEL) is sent while logged out.  So
	it is here we display our login dialog box and setup Profile context,
	authentication, etc.

	@bug	Re-loading the plugin each time... good or bad?
*/
int
WINAPI
WlxLoggedOutSAS (PVOID                pWlxContext,
				 DWORD                dwSasType,
				 PLUID                pAuthenticationId,
				 PSID                 pLogonSid,
				 PDWORD               pdwOptions,
				 PHANDLE              phToken,
				 PWLX_MPR_NOTIFY_INFO pMprNotifyInfo,
				 PVOID *              pProfile)
{
	int iRet;
	int attempt_count = 0;
	LPTSTR domainToUse;
	
	PGinaContext pGinaContext=(PGinaContext)pWlxContext;
	PGINA_CONTEXT pgContext = (PGINA_CONTEXT) pWlxContext;
	WLX_PROFILE_V2_0 *pWLXProfile;


	settingsInfo.pathProfile = NULL; 
    settingsInfo.mapPaths = TEXT("");
	settingsInfo.pathMSGina = TEXT("MSGINA.DLL");
	settingsInfo.pathProfile = NULL;
	settingsInfo.pathPlugin = TEXT("C:\\PGINA\\PLUGINS\\DUMMYPLUGIN.DLL");

BadPlugin:
    LoadSettings();

	if(!settingsInfo.disabled)
	{
		EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("Loading plugin %s"),settingsInfo.pathPlugin);
		if(!LoadPlugin()) {
            // Failed to load the plugin, so we set ourselves as bypass, this way the user can still get in
            regWriteBool(true,_T("passThru"));
            goto BadPlugin;
        }
		EventLog(false,EVENTLOG_INFORMATION_TYPE,TEXT("Plugin information: %s"),pluginAboutPlugin());

		// First look to see how we should deal with autologin
		if(honorAutoLogin) {
			usingAutoLogin = isAutoAdminValid();

			// If we are supposed to honor it, and the auto stuff is valid, go ahead and just fall back on MSGINA to do the work for us
			if(usingAutoLogin) {
                EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("AutoAdmin information is legit and we are supposed to honor it, passing through"));
				return pfWlxLoggedOutSAS(pWlxContext,dwSasType,pAuthenticationId,pLogonSid,pdwOptions,phToken,pMprNotifyInfo,pProfile);  
			}
		}

		/*for(;;)
		{
reshowlogin: */
			if(windowsVersion == WINDOWS_NT_4)
			{
				// It would be nice to use the WLX functions here... wonder why they cause BSOD in NT4?
				iRet = DialogBox(pGinahInstance,MAKEINTRESOURCE(IDD_LOGON_BOX),NULL,(DLGPROC)LoginBoxProc);    
			}
			else if(windowsVersion == WINDOWS_2000)
				iRet = ((PWLX_DISPATCH_VERSION_1_3)g_pWinlogon)->WlxDialogBox(HGLOBALWLX, pGinahInstance, MAKEINTRESOURCEW(IDD_LOGON_BOX), NULL, LoginBoxProc);
			else // Windows XP
				iRet = ((PWLX_DISPATCH_VERSION_1_4)g_pWinlogon)->WlxDialogBox(HGLOBALWLX, pGinahInstance, MAKEINTRESOURCEW(IDD_LOGON_BOX), NULL, LoginBoxProc); 

			EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("Login dialog returned: %d (0x%08x)"),iRet,iRet);

            if(iRet == LOGIN_SHUTDOWNREBOOT)
			{
				if(buttonAction)
					return WLX_SAS_ACTION_SHUTDOWN_REBOOT;
				else
					return WLX_SAS_ACTION_SHUTDOWN_POWER_OFF;
			}

			if(iRet == WLX_DLG_SCREEN_SAVER_TIMEOUT || iRet == WLX_DLG_INPUT_TIMEOUT) {
				return WLX_SAS_ACTION_NONE;
			}

			if(iRet == WLX_SAS_ACTION_LOGON)
			{
				// If we are NOT doing single user login (normal)
				if(!sUserLogin)
				{
					// If they are a plugin user, or if we must manage domain accounts
					if(settingsInfo.authType == AUTH_PLUGIN || (settingsInfo.authType == AUTH_DOMAIN && manageDomain))
					{
						EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("Creating user account."));
						if(!CreateLocalAccount())
						{
							// "An unknown error has prevented your account from being created.\n\rThis may be due to policy or security settings as well as other machine configuration.\n\rPlease consult your administrator."
							//pGinaMessageBox(IDS_CREATEACCOUNT,(UINT)0,MB_OK);
							MessageBox(GetForegroundWindow(),TranslatedString(GSECTION,TEXT("IDS_CREATEACCOUNT")),NULL,MB_ICONERROR | MB_OK);
							return WLX_SAS_ACTION_NONE;
						}
					}

					if(settingsInfo.authType == AUTH_DOMAIN && reqDom) {
						// Make sure we add the user to groups when the reqDom setting is on
						 AddUserToGroups();
					}

					if(!hookWlx) {
						SetAutoAdmin(1);
						SetAutoCredentials(settingsInfo.Username,settingsInfo.Password);
					} else {
						useSLoginCredentials = false;
					}
				}
				else if(settingsInfo.authType == AUTH_PLUGIN) // Only do single user for plugin auth'd users
				{
					// In this case Single User is on, log them in locally (or via domain)
					if(!hookWlx) {
						SetAutoAdmin(1);
						SetAutoCredentials(sUsername,sPassword);
					} else {
						useSLoginCredentials = true;
					}

					// We need to determine whether to login via the domain, or the local user, for now, if 
					//  includeDomain is on, assume AUTH_DOMAIN
					if(includeDomain)
						settingsInfo.authType = AUTH_DOMAIN;
					else
						settingsInfo.authType = AUTH_LOCAL;
				}
				else
				{
					// AUTH_LOCAL + sUserLogin
					if(!hookWlx) {
						SetAutoAdmin(1);
						SetAutoCredentials(settingsInfo.Username,settingsInfo.Password);
					} else {
						// We need to know that when we hook the wlx, which credentials to use
						useSLoginCredentials = false;
					}
				}

				if(includeDomain || manageDomain)
				{
					if(settingsInfo.defaultDomain)
						domainToUse = settingsInfo.defaultDomain;
					else
						domainToUse	= defaultDomain;
				}
				else
					domainToUse	= NULL;


				// What the heck.. lets give it a shot
				pMprNotifyInfo->pszUserName = _tcsdup(settingsInfo.Username);
				pMprNotifyInfo->pszPassword = _tcsdup(settingsInfo.Password);
				pMprNotifyInfo->pszDomain = domainToUse;
				pMprNotifyInfo->pszOldPassword = NULL;

				EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("Preperation complete.  Calling WlxLoggedOutSAS in %s"),settingsInfo.pathMSGina);

				iRet = pfWlxLoggedOutSAS(pWlxContext,
										 dwSasType,
										 pAuthenticationId,
										 pLogonSid,
										 pdwOptions,
										 phToken,
										 pMprNotifyInfo,
										 pProfile);  

				// Duplicate user token for drive mapping..
				if(!DuplicateTokenEx(*phToken, 
									 TOKEN_ALL_ACCESS,
									 NULL,
									 SecurityImpersonation,
									 TokenPrimary,
									 &hUser))
				{
					EventLog(false,EVENTLOG_ERROR_TYPE,TEXT("User Token duplication failed"));
				}
				else
				{
					DuplicateTokenEx(*phToken, 
									 TOKEN_ALL_ACCESS,
									 NULL,
									 SecurityImpersonation,
									 TokenPrimary,
									 &settingsInfo.hUser);
				}

				if(iRet != WLX_SAS_ACTION_LOGON)
					EventLog(false,EVENTLOG_ERROR_TYPE,TEXT("An error or non-handled response was returned from the stubbed GINA."));
				else {					
					// Tell Windows where the roaming profile is
					if (settingsInfo.authType != AUTH_LOCAL && settingsInfo.pathProfile) {
						pWLXProfile = (WLX_PROFILE_V2_0 *) LocalAlloc(LMEM_FIXED, sizeof(WLX_PROFILE_V2_0));
						
						pWLXProfile->dwType = WLX_PROFILE_TYPE_V2_0;
						pWLXProfile->pszProfile = wlxsdup(settingsInfo.pathProfile);
						pWLXProfile->pszPolicy = NULL;
						pWLXProfile->pszNetworkDefaultUserProfile = wlxsdup(settingsInfo.pathProfileDefault);
						pWLXProfile->pszServerName = NULL;
						pWLXProfile->pszEnvironment = NULL;
						*pProfile = pWLXProfile;
					}
				}
				if(!hookWlx) {
					SetAutoAdmin(0); // We are done, and done and dont want the pass in the reg
				}
				isLoggedIn = TRUE;

				logInTime = time(&logInTime);	// Keep our login time
				if ( settingsInfo.sessionTimeout != 0 ){
					sessionTimeLeft = settingsInfo.sessionTimeout;	// If the plugin set a session time
					TimerID = SetTimer( NULL , 0 , 1000, onTimerProc ); // Start our timer
				}else{
					TimerID = 0;				// no timeout, no timer
					sessionTimeLeft = 0;
				}

                if(saveLastUsername) {
                    regWriteString(settingsInfo.Username,_T("lastUserName"));
                }
                loginAttemptCount = 0;
				return iRet;
			}

            if(++loginAttemptCount > 3) {
                loginAttemptCount = 0;
                Sleep(5000);
            }
			/* attempt_count++;
			if(attempt_count < 3) {
				goto reshowlogin;
			} else {
				Sleep(5000);
				// Default to nothing
				return WLX_SAS_ACTION_NONE;
			}*/
            return WLX_SAS_ACTION_NONE;
		//}
		//return iRet;

	}
	else
	{
		EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("Passthru mode enabled."));

		iRet = pfWlxLoggedOutSAS(pWlxContext,
								 dwSasType,
								 pAuthenticationId,
								 pLogonSid,
								 pdwOptions,
								 phToken,
								 pMprNotifyInfo,
								 pProfile); 

				return iRet;
	}
}

/**
	@brief	Called after a successful login attempt to setup users context
	@author	Nathan Yocom
	@note	Profile handling may have to occur before passing control to MS?
	
	Passes control to MSGina in order to correctly setup context, then calls
	some support functions 
*/
BOOL
WINAPI
WlxActivateUserShell (PVOID pWlxContext,
					  PWSTR pszDesktopName,
					  PWSTR pszMprLogonScript,
					  PVOID pEnvironment)
{
	BOOL result = true;
	PGINA_CONTEXT pgContext = (PGINA_CONTEXT) pWlxContext;
	HKEY key;
	DWORD size = SECURITY_MAX_SID_SIZE;
	PSID everyoneSid, userSid;
	SID_IDENTIFIER_AUTHORITY sidAuthWorld = SECURITY_WORLD_SID_AUTHORITY;


	EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("Entered WlxActivateUserShell()"));
	
	if(!settingsInfo.disabled && (settingsInfo.authType == AUTH_PLUGIN || mapAdmin || sUserLogin))
	{
		if (settingsInfo.pathProfile && settingsInfo.authType == AUTH_PLUGIN && roamingProfile) {
			// Set the proper permissions for the roaming user's registry. This is necessary since the
			// the current user's SID may differ from the SID they had when they last logged on
			// (e.g. they logged on to a different machine, or pGina is configured to delete the
			// local account on logout). 
			key = RegOpenUser(settingsInfo.hUser, READ_CONTROL | WRITE_DAC);
			userSid = GetUserSid(settingsInfo.hUser);
			RegAddKeyPermission(key, userSid, GENERIC_ALL);
			

			// Remove the group Everyone from the user's registry permissions if it exists (this happens
			// when a new user gets a copy of the default profile)
			if (AllocateAndInitializeSid(&sidAuthWorld, 1,
                     SECURITY_WORLD_RID,
                     0, 0, 0, 0, 0, 0, 0,
                     &everyoneSid)) 
			{
				RegRemoveKeyPermission(key, everyoneSid);
				FreeSid(everyoneSid);
			}
			LocalFree(userSid);
			RegCloseKey(key);
		}
	}

	if(!settingsInfo.disabled && (settingsInfo.authType == AUTH_PLUGIN || settingsInfo.authType == AUTH_DOMAIN || mapAdmin || sUserLogin)) {
		MapDrives();
	}


	result =  pfWlxActivateUserShell(pWlxContext,pszDesktopName,pszMprLogonScript,pEnvironment);

	if(!settingsInfo.disabled)
	{
		pluginLoginHook(&settingsInfo);
		if (settingsInfo.authType == AUTH_PLUGIN || settingsInfo.authType == AUTH_DOMAIN || mapAdmin)
			runPostLoginScript(pszDesktopName);
	}

	EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("Leaving WlxActivateUserShell()"));

	return result;
}

/**
	@brief	Called when a user presses CTRL+ALT+DEL while logged in
	@author	Nathan Yocom
	@note	Currently only allows or prevents workstation locking
	
	If not in bypass mode, this function checks the settings in the registry
	and allows (or denies) the ability to lock the workstation
*/
int
WINAPI
WlxLoggedOnSAS (PVOID pWlxContext,
				DWORD dwSasType,
				PVOID pReserved)
{
	int retVal;
	PGinaContext pGinaContext=(PGinaContext)pWlxContext;

	if(!settingsInfo.disabled)
	{
        //desktopLocked = true;

		LoadSettings();

		/* retVal = pfWlxLoggedOnSAS(pWlxContext, 
							 dwSasType, 
							 pReserved); */

		blockSASmessages = TRUE;

        if(timedOutLogoff) {
            timedOutLogoff = false;
            return WLX_SAS_ACTION_LOGOFF;
        }
        if(idleLockHit) {
            idleLockHit = false;
            return WLX_SAS_ACTION_LOCK_WKSTA;
        }

		if(windowsVersion == WINDOWS_NT_4){
 			retVal = DialogBox(pGinahInstance,MAKEINTRESOURCE(IDD_LOCKDIAG),NULL,(DLGPROC)LockedBoxProc);    
		}else if(windowsVersion == WINDOWS_2000){
 			retVal = ((PWLX_DISPATCH_VERSION_1_3)g_pWinlogon)->WlxDialogBox(HGLOBALWLX,
 																			pGinahInstance,
 																			MAKEINTRESOURCEW(IDD_LOCKDIAG),
 																			NULL,
 																			LockedBoxProc);
		}else{
			retVal = ((PWLX_DISPATCH_VERSION_1_4)g_pWinlogon)->WlxDialogBox(HGLOBALWLX,
																			pGinahInstance,
																			MAKEINTRESOURCEW(IDD_LOCKDIAG),
																			NULL,
																			LockedBoxProc);
		}
		blockSASmessages = FALSE;

		if((retVal == WLX_SAS_ACTION_LOCK_WKSTA) && !allowLocking)
		{
			if(settingsInfo.isAdmin && regReadBool(TEXT("allowAdminLocks"))) {
				desktopLocked = true;
				return WLX_SAS_ACTION_LOCK_WKSTA;
			} else {
				MessageBox(GetForegroundWindow(),TranslatedString(GSECTION,TEXT("IDS_NOLOCK")),TranslatedString(GSECTION,TEXT("IDS_T_INVSELECTION")),MB_OK);
				return WLX_SAS_ACTION_NONE;
			}
		}

		if((retVal == WLX_SAS_ACTION_LOCK_WKSTA) && allowLocking) {
            desktopLocked = true;
			return WLX_SAS_ACTION_LOCK_WKSTA;
        }
        desktopLocked = false;
		return retVal;

	}
	else
	{
		return pfWlxLoggedOnSAS(pWlxContext, 
								dwSasType, 
								pReserved);
	}
}


VOID
WINAPI
WlxDisplayLockedNotice (PVOID pWlxContext)
{
	pfWlxDisplayLockedNotice(pWlxContext);
}


BOOL
WINAPI
WlxIsLockOk (PVOID pWlxContext)
{
	if(!settingsInfo.disabled && !allowLocking)
	{
		return FALSE;
	}
	else
		return pfWlxIsLockOk(pWlxContext);
}


int
WINAPI
WlxWkstaLockedSAS (PVOID pWlxContext,
				   DWORD dwSasType)
{
	PGinaContext pGinaContext=(PGinaContext)pWlxContext;
	int iRet = WLX_SAS_ACTION_UNLOCK_WKSTA;	// Default to unlocking
    int msRet = WLX_SAS_ACTION_UNLOCK_WKSTA;
	int auth_tries = 0;

	if(timedOutLogoff) {
		timedOutLogoff = false;
		return WLX_SAS_ACTION_LOGOFF;
	}

	if(!trueUnlock)
	{
		msRet = pfWlxWkstaLockedSAS(pWlxContext, dwSasType);
        if(msRet == WLX_SAS_ACTION_UNLOCK_WKSTA) {
            desktopLocked = false;
        }
        return msRet;
	}
	else
	{
		LoadPlugin();    

		for(;;)
		{
retryauth:
			if(windowsVersion == WINDOWS_NT_4)
				iRet = DialogBox(pGinahInstance,MAKEINTRESOURCE(IDD_LOCKED_BOX),NULL,(DLGPROC)LockedLoginBoxProc);
			else if(windowsVersion == WINDOWS_2000)
				iRet = ((PWLX_DISPATCH_VERSION_1_3)g_pWinlogon)->WlxDialogBox(HGLOBALWLX, pGinahInstance, MAKEINTRESOURCEW(IDD_LOCKED_BOX), NULL, LockedLoginBoxProc);
			else // Windows XP
				iRet = ((PWLX_DISPATCH_VERSION_1_4)g_pWinlogon)->WlxDialogBox(HGLOBALWLX, pGinahInstance, MAKEINTRESOURCEW(IDD_LOCKED_BOX), NULL, LockedLoginBoxProc);

			switch(iRet)
			{
            case WLX_SAS_ACTION_UNLOCK_WKSTA:
            case WLX_SAS_ACTION_FORCE_LOGOFF:
                desktopLocked = false;
				return iRet;
				break;
            case WLX_SAS_TYPE_TIMEOUT:
			case LOGIN_CANCELED:
				return WLX_SAS_ACTION_NONE;
				break;
			case LOGIN_FAILED:
				auth_tries++;
				if(auth_tries < 3) {
					goto retryauth;
				} else {
					Sleep(5000);
					return WLX_SAS_ACTION_NONE;
				}
				break;

			}
		}
	}

	return iRet;
}


BOOL
WINAPI
WlxIsLogoffOk (PVOID pWlxContext)
{
	BOOL bSuccess;
	PSID sid;
	HKEY key = NULL;

	bSuccess = pfWlxIsLogoffOk(pWlxContext);

	if(bSuccess)
	{
		if (!settingsInfo.disabled && settingsInfo.authType == AUTH_PLUGIN && !keepProfiles && roamingProfile)
		{
			key = RegOpenUser(settingsInfo.hUser, READ_CONTROL | WRITE_DAC);
			sid = GetUserSid(settingsInfo.hUser);
			RegRemoveKeyPermission(key, sid);
			RegCloseKey(key);
			LocalFree(sid);
		}

		//
		// If it's OK to logoff, make sure stored credentials are cleaned up.
		//
	}

	return bSuccess;
}


VOID
WINAPI
WlxLogoff (PVOID pWlxContext)
{
	SetAutoAdmin(0);

	EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("In Logoff()"));

	pfWlxLogoff(pWlxContext);

	EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("Logoff finished.."));

	// Find the users profile directory and remove it, then the account 
	if(!settingsInfo.disabled && (settingsInfo.authType == AUTH_PLUGIN || settingsInfo.authType == AUTH_DOMAIN) && (!keepProfiles || !keepAccount))
	{
		EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("DeleteAccount()"));
		DeleteLocalAccount(!keepAccount);
		EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("Done"));
	}


	if(!settingsInfo.disabled && 
	   settingsInfo.authType == AUTH_PLUGIN && 
	   keepProfiles && 
	   defaultDomain == NULL && 
	   settingsInfo.defaultDomain == NULL && 
	   scramblePasswords)
	{
		// Scramble the users password
		EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("Scramble pass..."));
		scramblePassword();
	}

	/*  if(!settingsInfo.disabled && settingsInfo.authType == AUTH_PLUGIN && keepProfiles)
		  pGinaSaveProfile(&settingsInfo); */


	if(!settingsInfo.disabled)
	{
		EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("LogoutHook"));
		pluginLogoutHook(&settingsInfo);
	}

	if(!settingsInfo.disabled && (settingsInfo.authType == AUTH_PLUGIN || settingsInfo.authType == AUTH_DOMAIN || mapAdmin || sUserLogin)) {
		DeMapDrives();
	}

	EventLog(true,EVENTLOG_INFORMATION_TYPE,TEXT("Out Logoff()"));

	logInTime = 0;
	sessionTimeLeft = 0;
	if ( TimerID != 0 ) KillTimer(NULL,TimerID);
	isLoggedIn = FALSE;
}


VOID
WINAPI
WlxShutdown(PVOID pWlxContext,
			DWORD ShutdownType)
{
	SetAutoAdmin(0);

	if(settingsInfo.disabled)
		pfWlxShutdown(pWlxContext, ShutdownType);

	if(isLoggedIn && !settingsInfo.disabled)
	{
		// Force a logoff first, then shutdown
		if(WlxIsLogoffOk(pWlxContext))
		{
			WlxLogoff(pWlxContext);
			pfWlxShutdown(pWlxContext, ShutdownType);
		}
	}
	else if(!settingsInfo.disabled)
		pfWlxShutdown(pWlxContext, ShutdownType); // Just straight shutdown
}


//
// New for version 1.1
//

BOOL
WINAPI
WlxScreenSaverNotify (PVOID  pWlxContext,
					  BOOL * pSecure)
{
    PGinaContext pGinaContext=(PGinaContext)pWlxContext;
    PGINA_CONTEXT pgContext = (PGINA_CONTEXT) pWlxContext;
    BOOL retval;
    int whatToDo = 0;

    retval = pfWlxScreenSaverNotify(pWlxContext, pSecure); 

	idleInterrupted = false;

    if(idleLogoff && isLoggedIn && !settingsInfo.disabled && (!desktopLocked || idleFromLocked)) {
        // Switch to the winlogon desktop
		// We're already there if the desktop is locked, and we dont need to do this if we are interrupted
		if(!desktopLocked) {
			if(windowsVersion == WINDOWS_2000){
				((PWLX_DISPATCH_VERSION_1_3)g_pWinlogon)->WlxSwitchDesktopToWinlogon(hGlobalWlx);
			}else{
				((PWLX_DISPATCH_VERSION_1_4)g_pWinlogon)->WlxSwitchDesktopToWinlogon(hGlobalWlx);
			}
		}
reshowDialog:
    
        // Let the user know their about to be logged off, let them start the screensaver if they want, or cancel
        //  and return to the screen.. otherwise with no input the user gets logged off
        // Note that we use DialogBox here because we dont want Winlogon to timeout the dialog on us
        if(windowsVersion == WINDOWS_2000)
            whatToDo = ((PWLX_DISPATCH_VERSION_1_3)g_pWinlogon)->WlxDialogBox(HGLOBALWLX, pGinahInstance, MAKEINTRESOURCEW(IDD_IDLELOGOUT), NULL, IdleBoxProc);
        else // Windows XP
            whatToDo = ((PWLX_DISPATCH_VERSION_1_4)g_pWinlogon)->WlxDialogBox(HGLOBALWLX, pGinahInstance, MAKEINTRESOURCEW(IDD_IDLELOGOUT), NULL, IdleBoxProc); 

		if(whatToDo == WLX_DLG_SCREEN_SAVER_TIMEOUT || whatToDo == WLX_DLG_INPUT_TIMEOUT) {
            idleInterrupted = true;
			goto reshowDialog;
		}

		 // Dont go back to the user desktop if the desktop was locked
        if(!desktopLocked && !idleInterrupted) {        
            if(windowsVersion == WINDOWS_2000){
                ((PWLX_DISPATCH_VERSION_1_3)g_pWinlogon)->WlxSwitchDesktopToUser(hGlobalWlx);
            }else{
                ((PWLX_DISPATCH_VERSION_1_4)g_pWinlogon)->WlxSwitchDesktopToUser(hGlobalWlx);
            }
        }

       if(whatToDo == IDLE_CANCEL) {
            return false;
        } else {
            timedOutLogoff = true;
			//if(!desktopLocked) {
				if(windowsVersion == WINDOWS_2000){
					((PWLX_DISPATCH_VERSION_1_3)g_pWinlogon)->WlxSasNotify(hGlobalWlx,WLX_SAS_TYPE_CTRL_ALT_DEL);
				}else{
					((PWLX_DISPATCH_VERSION_1_4)g_pWinlogon)->WlxSasNotify(hGlobalWlx,WLX_SAS_TYPE_CTRL_ALT_DEL);
				}       
			//}
            return false;
        }
    }

	return retval;
}

BOOL
WINAPI
WlxStartApplication (PVOID pWlxContext,
					 PWSTR pszDesktopName,
					 PVOID pEnvironment,
					 PWSTR pszCmdLine)
{
	BOOL retval = pfWlxStartApplication(pWlxContext,pszDesktopName,pEnvironment,pszCmdLine); 

	return retval;
}


//
// New for version 1.3
//

BOOL
WINAPI
WlxNetworkProviderLoad (PVOID                pWlxContext,
						PWLX_MPR_NOTIFY_INFO pNprNotifyInfo)
{
	return pfWlxNetworkProviderLoad(pWlxContext, pNprNotifyInfo);
}


BOOL
WINAPI
WlxDisplayStatusMessage (PVOID pWlxContext,
						 HDESK hDesktop,
						 DWORD dwOptions,
						 PWSTR pTitle,
						 PWSTR pMessage)
{
	return pfWlxDisplayStatusMessage(pWlxContext,
									 hDesktop,
									 dwOptions,
									 pTitle,
									 pMessage);
}


BOOL
WINAPI
WlxGetStatusMessage (PVOID   pWlxContext,
					 DWORD * pdwOptions,
					 PWSTR   pMessage,
					 DWORD   dwBufferSize)
{
	return pfWlxGetStatusMessage(pWlxContext,
								 pdwOptions,
								 pMessage,
								 dwBufferSize);
}


BOOL
WINAPI
WlxRemoveStatusMessage (PVOID pWlxContext)
{
	return pfWlxRemoveStatusMessage(pWlxContext);
}

VOID
WINAPI
WlxReconnectNotify (
				   PVOID                   pWlxContext
				   )
{
	pfWlxReconnectNotify(pWlxContext);
}

VOID
WINAPI
WlxDisconnectNotify (
					PVOID                   pWlxContext
					)
{
	pfWlxDisconnectNotify(pWlxContext);
}

BOOL WINAPI WlxGetConsoleSwitchCredentials( PVOID  pWlxContext,
											PVOID  pCredInfo )
{
	return pfWlxGetConsoleSwitchCredentials( pWlxContext, pCredInfo );
}


